@@ -33,7 +33,7 @@ public:
 	enum DoneAction { DoNothing=0, Load=1 };
 
 /** creates a new RKSaveAgent. If when_done == Quit, the RKSaveAgent will quit the application as soon as saving was successful (or it asked to by the user). Similarly, if when_done==Load, it will load a new workspace after saving (specify the url in load_url). If url is given (not empty), and not save_file_as, the agent will try to save to the given url, else it will ask the user to specify a url. RKSaveAgent will self destruct when done. */
-	explicit RKSaveAgent (KURL url, bool save_file_as=false, DoneAction when_done=DoNothing, KURL load_url=QString::null);
+	explicit RKSaveAgent (KUrl url, bool save_file_as=false, DoneAction when_done=DoNothing, KUrl load_url=KUrl());
 	
 	~RKSaveAgent ();
 protected:
@@ -42,8 +42,8 @@ private:
 	bool askURL ();
 	void done ();
 	RCommandChain *save_chain;
-	KURL save_url;
-	KURL load_url;
+	KUrl save_url;
+	KUrl load_url;
 	DoneAction when_done;
 };
 
