@@ -2,7 +2,7 @@
                           main.cpp  -  description
                              -------------------
     begin                : Tue Oct 29 20:06:08 CET 2002
-    copyright            : (C) 2002, 2005, 2006, 2007, 2008 by Thomas Friedrichsmeier 
+    copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier 
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -59,9 +59,15 @@
 #include "rkward.h"
 #include "rkwardapplication.h"
 
+#ifdef Q_WS_WIN
+	// these are needed for the exit hack.
+#	include <stdio.h>
+#	include <windows.h>
+#endif
+
 #include "debug.h"
 
-#include "../version.h"
+#include "version.h"
 
 int RK_Debug_Level = 0;
 int RK_Debug_Flags = ALL;
@@ -70,13 +76,14 @@ int RK_Debug_CommandStep = 0;
 static KCmdLineOptions options;
 
 int main(int argc, char *argv[]) {
-	options.add ("+[File]", ki18n ("file to open"), 0);
+	options.add ("evaluate <Rcode>", ki18n ("After starting (and after loading the specified workspace, if applicable), evaluate the given R code."), 0);
 	options.add ("debug-level <level>", ki18n ("Verbosity of debug messages (0-5)"), "2");
 	options.add ("debug-flags <flags>", ki18n ("Mask for components to debug (see debug.h)"), "8191");
 	options.add ("debugger <command>", ki18n ("Debugger (enclose any debugger arguments in single quotes ('') together with the command)"), "");
 	options.add ("disable-stack-check", ki18n ("Disable R C stack checking"), 0);
+	options.add ("+[File]", ki18n ("R workspace file to open"), 0);
 
-	KAboutData aboutData("rkward", QByteArray (), ki18n ("RKWard"), VERSION, ki18n ("Frontend to the R statistics language"), KAboutData::License_GPL, ki18n ("(c) 2002, 2004, 2005, 2006, 2007, 2008"), KLocalizedString (), "http://rkward.sf.net", "rkward-devel@lists.sourceforge.net");
+	KAboutData aboutData("rkward", QByteArray (), ki18n ("RKWard"), VERSION, ki18n ("Frontend to the R statistics language"), KAboutData::License_GPL, ki18n ("(c) 2002, 2004, 2005, 2006, 2007, 2008, 2009"), KLocalizedString (), "http://rkward.sf.net", "rkward-devel@lists.sourceforge.net");
 	aboutData.addAuthor (ki18n ("%1").subs ("Thomas Friedrichsmeier"), ki18n ("Project leader / main developer"));
 	aboutData.addAuthor (ki18n ("%1").subs ("Pierre Ecochard"), ki18n ("C++ coder since 0.2.9"));
 	aboutData.addAuthor (ki18n ("%1").subs ("Stefan Roediger"), ki18n ("Many plugins, suggestions, marketing, translations"));
@@ -91,15 +98,21 @@ int main(int argc, char *argv[]) {
 	aboutData.addCredit (ki18n ("%1").subs ("David Sibai"), ki18n ("Several valuable comments, hints and patches"));
 	aboutData.addCredit (ki18n ("%1").subs ("Ilias Soumpasis"), ki18n ("Translation, Suggestions, plugins"));
 	aboutData.addCredit (ki18n ("%1").subs ("Ralf Tautenhahn"), ki18n ("Many comments, useful suggestions, and bug reports"));
+	aboutData.addCredit (ki18n ("%1").subs ("Jannis Vajen"), ki18n ("German Translation, bug reports"));
 	aboutData.addCredit (ki18n ("%1").subs ("Roland Vollgraf"), ki18n ("Some patches"));
+	aboutData.addCredit (ki18n ("%1").subs ("Roy Qu"), ki18n ("patches and helpful comments"));
+	aboutData.addCredit (ki18n ("%1").subs ("Meik Michalke"), ki18n ("Many plugins, suggestions"));
 	aboutData.addCredit (ki18n ("Many more people on rkward-devel@lists.sourceforge.net"), ki18n ("Sorry, if we forgot to list you. Please contact us to get added"));
 
 	// before initializing the commandline args, remove the ".bin" from "rkward.bin".
 	// This is so it prints "Usage rkward..." instead of "Usage rkward.bin...", etc.
-	// it seems safest to keep a copy, since the shell still owns argv[0]
-	char *argv_zero_copy = argv[0];
-	argv[0] = qstrdup (QString (argv_zero_copy).remove (".bin").toLocal8Bit ());
-	KCmdLineArgs::init( argc, argv, &aboutData );
+	// it seems safest to keep a copy, since the shell still owns argv
+	char *argv_copy[argc];
+	argv_copy[0] = qstrdup (QString (argv[0]).remove (".bin").replace (".exe", ".bat").toLocal8Bit ());
+	for (int i = 1; i < argc; ++i) {
+		argv_copy[i] = argv[i];
+	}
+	KCmdLineArgs::init (argc, argv_copy, &aboutData);
 	KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
 
 	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
@@ -114,6 +127,7 @@ int main(int argc, char *argv[]) {
 		stoptions->initial_url = KUrl (args->makeURL (args->arg (0).toLatin1()));
 	}
 	stoptions->no_stack_check = args->isSet ("disable-stack-check");
+	stoptions->evaluate = args->getOption ("evaluate");
 
 	RKWardApplication app;
 	if (app.isSessionRestored ()) {
@@ -125,7 +139,20 @@ int main(int argc, char *argv[]) {
 
 	// do it!
 	int status = app.exec ();
-	// restore old argv[0] so the shell is happy
-	argv[0] = argv_zero_copy;
+#ifdef Q_WS_WIN
+	// HACK: Somehow, if we created a windows graph-device during runtime (possibly also on other conditions), we just can't exit cleanly anymore.
+	// We get out of the event loop, but once we return from main (including using _endthread(), _exit(), exit(), abort(), raise(SIGSEGV),
+	// ExitProcess(), and I don't know, what else I tried), the process just continues to sit there, hogging up one CPU.
+	// R 2.9.0, KDElibs 4.2.3, Qt 4.4.3, WinXP SP2
+	// So what we do is
+	// 1) Make extra sure, everything important is flushed
+	// 2) Use TerminateProcess () to kill all threads
+	// Alternatives: Use system ("taskkill ..."), but this command was not introduced before WinXP
+	app.flush();
+	app.processEvents();
+	_flushall ();
+
+	TerminateProcess (GetCurrentProcess (), 0);
+#endif
 	return status;
 }
