@@ -16,8 +16,8 @@
  ***************************************************************************/
 #include "getfilenamewidget.h"
 
-#include <qlayout.h>
 #include <qlabel.h>
+#include <QVBoxLayout>
 
 #include <klocale.h>
 #include <klineedit.h>
@@ -28,15 +28,16 @@
 GetFileNameWidget::GetFileNameWidget (QWidget *parent, FileType mode, const QString &label, const QString &caption, const QString &initial) : QWidget (parent) {
 	RK_TRACE (MISC);
 	QVBoxLayout *vbox = new QVBoxLayout (this);
-	vbox->setResizeMode (QLayout::Minimum);
+	vbox->setContentsMargins (0, 0, 0, 0);
+	vbox->setSizeConstraint (QLayout::SetMinimumSize);
 
 	vbox->addWidget (new QLabel (label, this));
 
-	edit = new KURLRequester (this);
+	edit = new KUrlRequester (this);
 	connect (edit, SIGNAL (textChanged (const QString &)), this, SLOT (locationEditChanged (const QString &)));
 	vbox->addWidget (edit);
 
-	edit->setURL (initial);
+	edit->setUrl (initial);
 	if (mode == ExistingDirectory) {
 		edit->setMode (KFile::Directory | KFile::ExistingOnly);
 	} else if (mode == ExistingFile) {
@@ -47,8 +48,8 @@ GetFileNameWidget::GetFileNameWidget (QWidget *parent, FileType mode, const QStr
 		RK_ASSERT (false);
 	}
 
-	if (caption.isEmpty ()) edit->setCaption (label);
-	else edit->setCaption (caption);
+	if (caption.isEmpty ()) edit->setWindowTitle (label);
+	else edit->setWindowTitle (caption);
 }
 
 GetFileNameWidget::~GetFileNameWidget () {
@@ -65,7 +66,7 @@ void GetFileNameWidget::setFilter (const QString &filter) {
 void GetFileNameWidget::setLocation (const QString &new_location) {
 	RK_TRACE (MISC);
 
-	edit->setURL (new_location);
+	edit->setUrl (new_location);
 }
 
 void GetFileNameWidget::locationEditChanged (const QString &) {
@@ -74,13 +75,15 @@ void GetFileNameWidget::locationEditChanged (const QString &) {
 }
 
 QString GetFileNameWidget::getLocation () {
-	return edit->url ();
+	return (edit->url ().path ());
 }
 
 void GetFileNameWidget::setBackgroundColor (const QColor & color) {
 	RK_TRACE (MISC);
 
-	edit->lineEdit ()->setBackgroundColor (color);
+	QPalette palette = edit->lineEdit ()->palette ();
+	palette.setColor (edit->lineEdit ()->backgroundRole (), color);
+	edit->lineEdit ()->setPalette (palette);
 }
 
 #include "getfilenamewidget.moc"
