@@ -2,7 +2,7 @@
                           rkstandardicons  -  description
                              -------------------
     begin                : Wed Oct 24 2007
-    copyright            : (C) 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -41,11 +41,14 @@ public:
 		ActionRunAll=0,
 		ActionRunLine,
 		ActionRunSelection,
+		ActionCDToScript,
 
 		ActionDeleteRow,
 		ActionInsertRow,
 		ActionDeleteVar,
 		ActionInsertVar,
+		ActionPasteInsideTable,
+		ActionPasteInsideSelection,
 
 		ActionAddRight,
 		ActionRemoveLeft,
