@@ -2,7 +2,7 @@
                           rkcomponent  -  description
                              -------------------
     begin                : Tue Dec 13 2005
-    copyright            : (C) 2005, 2006 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -17,6 +17,8 @@
 
 #include "rkcomponent.h"
 
+#include "../misc/rkcommonfunctions.h"
+
 #include "../debug.h"
 
 //############### RKComponentBase #####################
@@ -43,23 +45,26 @@ void RKComponentBase::addChild (const QString &id, RKComponentBase *child) {
 	child_map.insertMulti (id, child);		// no overwriting even on duplicate ("#noid#") ids
 }
 
-void RKComponentBase::fetchPropertyValuesRecursive (QMap<QString, QString> *list, bool include_top_level, const QString &prefix) {
+void RKComponentBase::fetchPropertyValuesRecursive (QMap<QString, QString> *list, bool include_top_level, const QString &prefix) const {
 	RK_TRACE (PLUGIN);
 
 	for (QHash<QString, RKComponentBase*>::const_iterator it = child_map.constBegin (); it != child_map.constEnd (); ++it) {
-		if (it.key () != "#noid#") {
-			if (it.value ()->isProperty ()) {
-				if (include_top_level) {
-					list->insert (prefix + it.key (), it.value ()->value ());
-				}
-			} else {
-				it.value ()->fetchPropertyValuesRecursive (list, true, prefix + it.key () + '.');
+		if (it.key () == "#noid#") continue;
+		if (it.value ()->isInternal ()) continue;
+
+		if (it.value ()->isProperty ()) {
+			if (include_top_level) {
+				list->insert (prefix + it.key (), static_cast<RKComponentPropertyBase*> (it.value ())->value ());
 			}
+		} else {
+			RK_ASSERT (it.value ()->isComponent ());
+			if (!static_cast<RKComponent *> (it.value ())->isEnabled ()) continue;
+			it.value ()->fetchPropertyValuesRecursive (list, true, prefix + it.key () + '.');
 		}
 	}
 }
 
-void RKComponentBase::setPropertyValues (QMap<QString, QString> *list) {
+void RKComponentBase::setPropertyValues (QMap<QString, QString> *list, bool warn_internal) {
 	RK_TRACE (PLUGIN);
 	// TODO: visibility enabledness and requiredness should be excluded, as those are not directly user settable. Perhaps even mark up all properties as user settable or not.
 
@@ -67,11 +72,54 @@ void RKComponentBase::setPropertyValues (QMap<QString, QString> *list) {
 		QString mod;
 		RKComponentBase *prop = lookupComponent (it.key (), &mod);
 		if (mod.isEmpty () && prop->isProperty ()) {		// found a property
-			static_cast<RKComponentPropertyBase*>(prop)->setValue (it.value ());
+			RKComponentPropertyBase* p = static_cast<RKComponentPropertyBase*>(prop);
+			RK_ASSERT (!(p->isInternal () && warn_internal));
+			p->setValue (it.value ());
 		}
 	}
 }
 
+QString RKComponentBase::serializeState () const {
+	RK_TRACE (PLUGIN);
+
+	QMap<QString, QString> props;
+	fetchPropertyValuesRecursive (&props, true);
+
+	QString out;
+	for (QMap<QString, QString>::const_iterator it = props.constBegin (); it != props.constEnd (); ++it) {
+		if (!out.isEmpty ()) out.append ("\n");
+		out.append (RKCommonFunctions::escape (it.key () + "=" + it.value ()));
+	}
+
+	return out;
+}
+
+RKComponent::UnserializeError RKComponentBase::unserializeState (const QStringList &state) {
+	RK_TRACE (PLUGIN);
+
+	QMap<QString, QString> props;
+
+	for (int i = 0; i < state.count (); ++i) {
+		QString line = state[i];
+		int sep = line.indexOf ('=');
+		if (sep < 0) return BadFormat;
+		props.insert (RKCommonFunctions::unescape (line.left (sep)), RKCommonFunctions::unescape (line.mid (sep+1)));
+	}
+
+	setPropertyValues (&props);
+
+	// verify
+	UnserializeError error = NoError;
+	for (QMap<QString, QString>::const_iterator it = props.constBegin (); it != props.constEnd (); ++it) {
+		if (fetchStringValue (it.key ()) != it.value ()) {
+			RK_DO(qDebug ("Tried to apply value %s to property %s, but got %s", qPrintable (it.value ()), qPrintable (it.key ()), qPrintable (fetchStringValue (it.key ()))), PLUGIN, DL_INFO);
+			error = NotAllSettingsApplied;
+		}
+	}
+
+	return error;
+}
+
 QString RKComponentBase::fetchStringValue (const QString &identifier) {
 	RK_TRACE (PLUGIN);
 
@@ -112,12 +160,15 @@ void RKComponent::createDefaultProperties () {
 
 	addChild ("enabled", enabledness_property = new RKComponentPropertyBool (this, false));
 	enabledness_property->setBoolValue (true);
+	enabledness_property->setInternal (true);
 	connect (enabledness_property, SIGNAL (valueChanged (RKComponentPropertyBase *)), this, SLOT (propertyValueChanged (RKComponentPropertyBase *)));
 	addChild ("visible", visibility_property = new RKComponentPropertyBool (this, false));
 	visibility_property->setBoolValue (true);
+	visibility_property->setInternal (true);
 	connect (visibility_property, SIGNAL (valueChanged (RKComponentPropertyBase *)), this, SLOT (propertyValueChanged (RKComponentPropertyBase *)));
 	addChild ("required", requiredness_property = new RKComponentPropertyBool (this, false));
 	requiredness_property->setBoolValue (true);
+	requiredness_property->setInternal (true);
 	connect (requiredness_property, SIGNAL (valueChanged (RKComponentPropertyBase *)), this, SLOT (propertyValueChanged (RKComponentPropertyBase *)));
 }
 
@@ -223,18 +274,18 @@ void RKComponent::removeFromParent () {
 
 	if (!parentComponent ()) return;
 
-	for (QHash<QString, RKComponentBase*>::const_iterator it = parentComponent ()->child_map.constBegin (); it != parentComponent ()->child_map.constEnd (); ++it) {
-		if (it.value () == this) {
-			QString key = it.key ();
 	// unfortunately, several items might hvae the same key, and there seems to be no way to selectively remove the current item only.
 	// however, this function should only ever be called in cases of emergency and to prevent crashes. So we make extra sure to remove the child,
 	// even if we remove a little more than necessary along the way.
-			while (parentComponent ()->child_map.remove (key));
-			return;
-		}
-	}
+	QString key = getIdInParent ();
+	while (parentComponent ()->child_map.remove (key)) {;}
+}
 
-	RK_ASSERT (false);
+QString RKComponent::getIdInParent () const {
+	RK_TRACE (PLUGIN);
+
+	if (!parentComponent ()) return QString ();
+	return (parentComponent ()->child_map.key (const_cast<RKComponent*> (this)));
 }
 
 #include "rkcomponent.moc"
