@@ -2,7 +2,7 @@
                           rkcomponent  -  description
                              -------------------
     begin                : Tue Dec 13 2005
-    copyright            : (C) 2005, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -28,7 +28,7 @@ class RKComponentPropertyBase;
 class RKComponentBase {
 public:
 /** constructor */
-	RKComponentBase () { required=true; };
+	RKComponentBase () { required=true; is_internal=false; };
 /** destructor */
 	virtual ~RKComponentBase () {};
 /** enum of types of properties. Used from RTTI. Don't change the values, as there are some range checks in the code (see isProperty ()) */
@@ -61,6 +61,12 @@ public:
 		ComponentContextHandler = 2900,
 		ComponentUser = 3000	/**< for user expansion */
 	};
+	enum UnserializeError {
+		NoError,
+		BadFormat,
+		NotAllSettingsApplied,
+		NoSuchComponent
+	};
 /** for RTTI. see RKComponentBase::RKComponentTypes */
 	virtual int type () = 0;
 /** tries to locate a component (or property) described by identifier as a child (of any generation) of this RKComponentBase. If found, a pointer to this is returned. Also, the modifier parameter is set to hold any remaining modifier contained in the identifier.
@@ -83,17 +89,29 @@ public:
 	void setRequired (bool require) { required = require; };
 /** simple convenience function to add a child to the map of children */
 	void addChild (const QString &id, RKComponentBase *child);
+
+/** serialize the state of this component / property and all its children. Note: Only the non-internal property-values are serialzed, not the components / properties themselves. @see fetchPropertyValuesRecursive() */
+	QString serializeState () const;
+/** set values from a string created with serializeState() (split by "\n"). @see serializeState (), @see setPropertyValues ().
+@returns status code */
+	UnserializeError unserializeState (const QStringList &state);
+
+/** Some properties/components will be marked as internal, such as visibility properties, which are not meant to be set directly by the user. These will be ignored in RKComponent::fetchPropertyValuesRecursive() */
+	void setInternal (bool internal) { is_internal = internal; };
+	bool isInternal () const { return is_internal; };
 protected:
 	QHash<QString, RKComponentBase*> child_map;
 	bool required;
-/** recursively fetch the current values of all properties present as direct or indirect children of this component. Used to transfer values e.g. when switching interfaces (or to store settings per plugin in the future). Values are placed in the dictionary provided (be sure to create one first!).
+/** recursively fetch the current values of all properties present as direct or indirect children of this component. Used to transfer values e.g. when switching interfaces (or to store settings per plugin in the future). Values are placed in the dictionary provided (be sure to create one first!). Internal properties are ignored (@see RKComponentPropertyBase::isInternal ());
 @param list the list to store the object values in
 @param include_top_level include direct properties of the component in the list (or only properties of children)
 @param prefix used during recursion to provide full ids for the added objects */
-	void fetchPropertyValuesRecursive (QMap<QString, QString> *list, bool include_top_level=false, const QString &prefix=QString::null);
+	void fetchPropertyValuesRecursive (QMap<QString, QString> *list, bool include_top_level=false, const QString &prefix=QString::null) const;
 /** counterpart to fetchPropertyValuesRecursive (). Tries to apply all values from the list to properties of the given names. If some keys can not be found, or do not resolve to properties, the are ignored.
 @param list a list of id->value such as generated by fetchPropertyValuesRecursive () */
-	void setPropertyValues (QMap<QString, QString> *list);
+	void setPropertyValues (QMap<QString, QString> *list, bool warn_internal=false);
+private:
+	bool is_internal;
 };
 
 #include "rkcomponentproperties.h"
@@ -145,10 +163,13 @@ public:
 	void setRequired (bool required);
 
 /** The parent of this component. Should be notified, whenever isSatisfied () or isReady ()-state changed. */
-	RKComponent *parentComponent () { return _parent; };
+	RKComponent *parentComponent () const { return _parent; };
 
 /** Is the component "ready"? I.e. it is up to date according to current settings. Does not imply it is also satisfied. Default implementation always returns true. */
 	virtual bool isReady () { return true; };
+
+/** Find the id of this component. NOTE: this is slow. Better to store the id in the first place, if needed */
+	QString getIdInParent () const;
 protected slots:
 /** if a child component self-destructs, it should remove itself from its parent *before* destructing. Don't use in a regular destructor. Call only if the child dies unexpectedly */
 	void removeFromParent ();
