@@ -2,7 +2,7 @@
                           rkcomponentmap.cpp  -  description
                              -------------------
     begin                : Thu May 12 2005
-    copyright            : (C) 2005, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,15 +22,17 @@
 
 #include <klocale.h>
 #include <kactioncollection.h>
+#include <kmessagebox.h>
 
 #include "rkcomponentcontext.h"
+#include "rkstandardcomponent.h"
 #include "../misc/xmlhelper.h"
 #include "../debug.h"
 #include "../rkglobals.h"
 #include "../rkward.h"
 
 QString RKPluginMapFile::makeFileName (const QString &filename) {
-	return QDir (basedir).filePath (filename);
+	return QDir::cleanPath (QDir (basedir).filePath (filename));
 }
 
 RKComponentGUIXML::RKComponentGUIXML () {
@@ -126,7 +128,7 @@ int RKComponentGUIXML::addSubMenu (QDomElement& parent, const QDomElement& descr
 }
 
 /////////////////////////// END RKComponentXMLGUIClient /////////////////////////////////
-////////////////////////////// BEGIN RKComponentMap /////////////////////////////////////
+////////////////////////////// Bhttp://apps.sourceforge.net/mediawiki/rkward/nfs/project/r/rk/rkward/6/6d/RKWardApplicationDetached.pngEGIN RKComponentMap /////////////////////////////////////
 
 // static members
 RKComponentMap *RKComponentMap::component_map = 0;
@@ -140,6 +142,9 @@ void RKComponentMap::initialize () {
 
 RKComponentMap::RKComponentMap () : RKComponentGUIXML (), KXMLGUIClient () {
 	RK_TRACE (PLUGIN);
+
+	setComponentData (KGlobal::mainComponent ());
+	actionCollection ()->setConfigGroup ("Plugin Shortcuts");
 }
 
 RKComponentMap::~RKComponentMap () {
@@ -229,6 +234,60 @@ QString RKComponentMap::getComponentIdLocal (RKComponentHandle* component) {
 	return (QString ());
 }
 
+//static
+bool RKComponentMap::invokeComponent (const QString &component_id, const QStringList &serialized_settings, ComponentInvocationMode submit_mode, QString *message, RCommandChain *in_chain) {
+	RK_TRACE (PLUGIN);
+
+	QString _message;
+	RKComponentHandle *handle = getComponentHandle (component_id);
+	if (!handle) {
+		_message = i18n ("You tried to invoke a plugin called '%1', but that plugin is currently unknown. Probably you need to load the corresponding PluginMap (Settings->Configure RKWard->Plugins), or perhaps the plugin was renamed.").arg (component_id);
+		if (message) *message = _message;
+		else KMessageBox::sorry (RKWardMainWindow::getMain (), _message, i18n ("No such plugin"));
+		return false;
+	}
+
+	RKStandardComponent *component = handle->invoke (0, 0);
+	RK_ASSERT (component);
+
+	RKComponent::UnserializeError error = component->unserializeState (serialized_settings);
+	if (error == RKComponent::BadFormat) {
+		_message = i18n ("Bad serialization format while trying to invoke plugin '%1'. Please contact the RKWard team (Help->About RKWard->Authors).").arg (component_id);
+		if (message) *message = _message;
+		else KMessageBox::error (component, _message, i18n ("Bad serialization format"));
+		return false;
+	}
+	if (error == RKComponent::NotAllSettingsApplied) {
+		_message = i18n ("Not all specified settings could be applied. Most likely this is because some R objects are no longer present in your current workspace.");
+		if (message) *message = _message;
+		else KMessageBox::information (component, _message, i18n ("Not all settings applied"));
+		// TODO: Don't show again-box?
+		// not considered an error
+	}
+
+	// Auto-Submit
+	if (submit_mode != ManualSubmit) {
+#ifndef Q_OS_WIN
+		// if the plugin takes longer than 5 seconds to settle, than that really is sort of buggy...
+		bool submit_ok = component->submit (5000, in_chain);
+#else
+#warning Temporary workaround. Remove this.
+		// (yet on windows, the PHP backend is *real* slow. We give it a bit longer as long as we still use it...
+		bool submit_ok = component->submit (50000, in_chain);
+#endif
+		if (submit_ok || (submit_mode == AutoSubmitOrFail)) component->close ();
+		if (!submit_ok) {
+			_message.append (i18n ("\nThe plugin could not be auto-submitted with these settings."));
+			if (message) *message = _message;
+			else KMessageBox::sorry (RKWardMainWindow::getMain (), _message, i18n ("Could not submit"));
+
+			return (submit_mode != AutoSubmitOrFail);
+		}
+	}
+
+	return true;
+}
+
 int RKComponentMap::addPluginMap (const QString& plugin_map_file) {
 	RK_TRACE (PLUGIN);
 
@@ -316,6 +375,7 @@ int RKComponentMap::addPluginMapLocal (const QString& plugin_map_file) {
 	}
 
 	setXMLGUIBuildDocument (gui_xml);
+	actionCollection ()->readSettings ();
 	return counter;
 }
 
@@ -323,7 +383,10 @@ void RKComponentMap::addedEntry (const QString &id, RKComponentHandle *handle) {
 	RK_TRACE (PLUGIN);
 
 	if (handle->isPlugin ()) {
-		actionCollection ()->addAction (id, handle, SLOT (activated()))->setText (handle->getLabel ());
+		handle->setAccessible (true);
+		KAction *action = actionCollection ()->addAction (id, handle, SLOT (activated()));
+		action->setText (handle->getLabel ());
+		action->setShortcutConfigurable (true);
 	}
 }
 
@@ -341,6 +404,7 @@ RKComponentHandle::RKComponentHandle (RKPluginMapFile *pluginmap, const QString
 	RKComponentHandle::plugin_map = pluginmap;
 
 	attributes = 0;
+	is_accessible = false;
 }
 
 RKComponentHandle::~RKComponentHandle () {
