@@ -2,7 +2,7 @@
                           rkcomponentproperties  -  description
                              -------------------
     begin                : Fri Nov 25 2005
-    copyright            : (C) 2005, 2006, 2007, 2008 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -172,6 +172,7 @@ RKComponentBase* RKComponentPropertyBool::lookupComponent (const QString &identi
 	if (next == "not") {
 		RKComponentPropertyBool *negated = new RKComponentPropertyBool (this, false, false, value_true, value_false);
 		negated->setInverted (true);
+		negated->setInternal (true);
 		negated->connectToGovernor (this);
 		*remainder = QString::null;		// reset
 		addChild ("not", negated);		// so subsequent lookups will not recreate the negated property
@@ -812,7 +813,7 @@ bool RKComponentPropertyRObjects::setValue (const QString &value) {
 	QStringList slist = value.split (separator, QString::SkipEmptyParts);
 
 	for (QStringList::const_iterator it = slist.begin (); it != slist.end (); ++it) {
-		RObject *obj = RObjectList::getObjectList ()->findObject (value);
+		RObject *obj = RObjectList::getObjectList ()->findObject (*it);
 		if (obj && isObjectValid (obj)) {
 			object_list.append (obj);
 			listenForObject (obj);
@@ -832,7 +833,7 @@ bool RKComponentPropertyRObjects::isStringValid (const QString &value) {
 	QStringList slist = value.split (separator, QString::SkipEmptyParts);
 
 	for (QStringList::const_iterator it = slist.begin (); it != slist.end (); ++it) {
-		RObject *obj = RObjectList::getObjectList ()->findObject (value);
+		RObject *obj = RObjectList::getObjectList ()->findObject (*it);
 		if (!(obj && isObjectValid (obj))) {
 			return false;
 		}
