@@ -2,7 +2,7 @@
                           rkstandardcomponent  -  description
                              -------------------
     begin                : Sun Feb 19 2006
-    copyright            : (C) 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -24,6 +24,7 @@
 #include <qtimer.h>
 #include <QVBoxLayout>
 #include <QGroupBox>
+#include <QTime>
 
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -61,12 +62,14 @@ RKStandardComponent::RKStandardComponent (RKComponent *parent_component, QWidget
 
 	RKStandardComponent::filename = filename;
 	RKStandardComponent::handle = handle;
+	command_chain = 0;
 	backend = 0;
 	gui = 0;
 	wizard = 0;
 	created = false;
 	addChild ("code", code = new RKComponentPropertyCode (this, true));		// do not change this name!
-	
+	code->setInternal (true);
+
 	// open the main description file for parsing
 	XMLHelper* xml = XMLHelper::getStaticHelper ();
 	QDomElement doc_element = xml->openXMLFile (filename, DL_ERROR);
@@ -307,6 +310,37 @@ void RKStandardComponent::buildAndInitialize (const QDomElement &doc_element, co
 	changed ();
 }
 
+bool RKStandardComponent::submit (int max_wait, RCommandChain *in_chain) {
+	RK_TRACE (PLUGIN);
+
+	RCommandChain *old_chain = command_chain; 	// should always be 0, but let's store it cleanly
+	command_chain = in_chain;
+	bool result = false;
+
+	QTime t;
+	t.start ();
+	while ((handle_change_timer->isActive () || backend->isBusy ()) && (t.elapsed () < max_wait)) {
+		QCoreApplication::processEvents (QEventLoop::ExcludeUserInputEvents, (max_wait / 2));
+	}
+	if (!(handle_change_timer->isActive () || backend->isBusy ())) {
+		if (isSatisfied ()) {
+			gui->ok ();
+			result = true;
+		}
+	}
+	command_chain = old_chain;
+	return result;
+}
+
+void RKStandardComponent::close () {
+	RK_TRACE (PLUGIN);
+
+	if (gui && (!parentComponent ())) {
+		QTimer::singleShot (0, gui, SLOT (close ()));
+	} else {
+		RK_ASSERT (false);
+	}
+}
 
 void RKStandardComponent::changed () {
 	RK_TRACE (PLUGIN);
@@ -627,6 +661,7 @@ void RKComponentBuilder::parseLogic (const QDomElement &element) {
 		QString id = xml->getStringAttribute (*it, "id", "#noid#", DL_WARNING);
 		RKComponentPropertyBase *prop = new RKComponentPropertyBase (component (), xml->getBoolAttribute (*it, "required", false, DL_INFO));
 		component ()->addChild (id, prop);
+		prop->setInternal (true);
 		component ()->connect (prop, SIGNAL (valueChanged (RKComponentPropertyBase *)), component (), SLOT (outsideValueChanged (RKComponentPropertyBase *)));
 
 		QString dummy = xml->getStringAttribute (*it, "default", QString::null, DL_INFO);
@@ -640,6 +675,7 @@ void RKComponentBuilder::parseLogic (const QDomElement &element) {
 	children = xml->getChildElements (element, "convert", DL_INFO);
 	for (it = children.constBegin (); it != children.constEnd (); ++it) {
 		RKComponentPropertyConvert *convert = new RKComponentPropertyConvert (component ());
+		convert->setInternal (true);
 		QString id = xml->getStringAttribute (*it, "id", "#noid#", DL_WARNING);
 		int mode = xml->getMultiChoiceAttribute (*it, "mode", convert->convertModeOptionString (), 0, DL_WARNING);
 		QString sources = xml->getStringAttribute (*it, "sources", QString::null, DL_WARNING);
