@@ -18,14 +18,15 @@
 #ifndef RKSTANDARDCOMPONENTGUI_H
 #define RKSTANDARDCOMPONENTGUI_H
 
-#include <qwidgetstack.h>
+#include <QStackedWidget>
+#include <QList>
 
 #include "rkstandardcomponent.h"
 
 /** For use in RKStandardComponents that are in wizard mode. Keeps a list of all pages, which page we're currently on, which children need to be satisfied in order to be able to move to the next page, etc.
 
 @author Thomas Friedrichsmeier */
-class RKStandardComponentStack: public QWidgetStack {
+class RKStandardComponentStack: public QStackedWidget {
 public:
 /** constructor. */
 	explicit RKStandardComponentStack (QWidget *parent);
@@ -46,17 +47,12 @@ public:
 	void addComponentToCurrentPage (RKComponent *component);
 private:
 /** pages are NOT the parent of their components (that would be theoretically possible, but a terrible mess, requiring a fully transparent type of RKComponent), hence we keep a manual list for each page */
-	typedef QValueList<RKComponent *> PageComponents;
+	typedef QList<RKComponent *> PageComponents;
 	struct PageDef {
 		PageComponents page_components;
 		RKComponent *page;
 	};
-	typedef QValueList<PageDef *> Pages;
-	Pages pages;
-
-	int num_pages;
-	int current_page;
-	PageDef *current_def;		// to save a few lookups
+	QList<PageDef*> pages;
 
 	int previousVisiblePage ();
 	int nextVisiblePage ();
@@ -68,6 +64,7 @@ class RKCommandEditorWindow;
 class QPushButton;
 class QTimer;
 class QSplitter;
+class QCloseEvent;
 
 /** contains the standard GUI elements for a top-level RKStandardComponent. The base class creates a dialog interface. For a wizard interface use RKStandardComponentWizard. You *must* call createDialog () after construction, since I can't virualize this for reasons I don't understand!
 
@@ -84,7 +81,7 @@ public:
 	virtual void enableSubmit (bool enable);
 	virtual void updateCode ();
 /** reimplemented from QWidget to take care of showing the code display if needed */
-	void show ();
+	void showEvent (QShowEvent *e);
 public slots:
 	void ok ();
 	void cancel ();
