@@ -7,7 +7,7 @@ function calculate () {
 	$q = "c (" . preg_replace ("/[, ]+/", ", ", getRK_val ("q")) . ")";
 
 ?>
-rk.temp = (pexp (q = <? echo ($q); ?>, rate = <? getRK ("rate"); ?>, <? getRK ("tail"); ?>, <? getRK("logp"); ?>))
+result <- (pexp (q = <? echo ($q); ?>, rate = <? getRK ("rate"); ?>, <? getRK ("tail"); ?>, <? getRK("logp"); ?>))
 <?
 }
 
@@ -15,13 +15,7 @@ function printout () {
 	global $q;
 ?>
 rk.header ("Exponential probabilities", list ("Vector of quantiles", "<? echo ($q); ?>", "Rate", "<? getRK ("rate"); ?>", "Tail", "<? getRK ("tail"); ?>", "Probabilities p are given as", "<? getRK ("logp"); ?>"))
-cat ("<h3>Exponential probabilities:  ", rk.temp, "</h3>")
-<?
-}
-
-function cleanup () {
-?>
-rm (rk.temp)
+rk.results (result, titles="Exponential probabilities")
 <?
 }
 ?>
