@@ -2,7 +2,7 @@
                           rembedinternal  -  description
                              -------------------
     begin                : Sun Jul 25 2004
-    copyright            : (C) 2004, 2005, 2006, 2007, 2008 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2005, 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -20,66 +20,55 @@
 // static
 REmbedInternal *REmbedInternal::this_pointer = 0; 
 
-#define TRUE true
-#define FALSE false
 #include <qstring.h>
+#include <QStringList>
 #include <qtextcodec.h>
+#include <klocale.h>
 #include "../core/robject.h"
 #include "../debug.h"
-#undef TRUE
-#undef FALSE
 
 #include "rklocalesupport.h"
 #include "rkpthreadsupport.h"
 #include "rksignalsupport.h"
 
-#include <dlfcn.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdint.h>
+#ifdef Q_WS_WIN
+#	include <winsock.h>
+#	undef ERROR	// windows define clashes with R define
+#else
+#include <dlfcn.h>
 #include <sys/resource.h>
 #include <sys/types.h>
-//#include <signal.h>
-//#include <unistd.h>
+#endif
 #include <math.h>
 
 #include "Rversion.h"
 
-#if (R_VERSION > R_Version(2, 2, 9))
-#define R_2_3
-#endif
-
-#if (R_VERSION > R_Version(2, 3, 9))
-#define R_2_4
-#endif
-
-#if (R_VERSION > R_Version(2, 4, 9))
-#define R_2_5
-#endif
-
-#if (R_VERSION > R_Version(2, 5, 9))
-#define R_2_6
-#endif
-
 #if (R_VERSION > R_Version(2, 6, 9))
 #define R_2_7
 #endif
 
+#ifndef R_2_7
+#error R version 2.7.0 or later is needed to compile this version of RKWard
+#endif
+
+#if (R_VERSION > R_Version(2, 8, 9))
+#define R_2_9
+#endif
+
 extern "C" {
 #define R_INTERFACE_PTRS 1
 
 // needed to detect CHARSXP encoding
-#ifdef R_2_7
-#	define IS_UTF8(x) (Rf_getCharCE(x) == CE_UTF8)
-#	define IS_LATIN1(x) (Rf_getCharCE(x) == CE_LATIN1)
-#else
-#	define USE_RINTERNALS 1
-#	define UTF8_MASK (1<<3)
-#	define IS_UTF8(x) ((x)->sxpinfo.gp & UTF8_MASK)
-#	define LATIN1_MASK (1<<2)
-#	define IS_LATIN1(x) ((x)->sxpinfo.gp & LATIN1_MASK)
+#define IS_UTF8(x) (Rf_getCharCE(x) == CE_UTF8)
+#define IS_LATIN1(x) (Rf_getCharCE(x) == CE_LATIN1)
+
+#ifdef Q_WS_WIN
+	// needed for R includes
+#	define Win32
 #endif
-// end
 
 #include "Rdefines.h"
 #include "R_ext/Rdynload.h"
@@ -87,106 +76,99 @@ extern "C" {
 #include "R_ext/Callbacks.h"
 #include "R.h"
 #include "Rinternals.h"
-#include "Rinterface.h"
-#include "Rdevices.h"
 #include "R_ext/Parse.h"
-
-#ifdef R_2_4
-#define USE_R_REPLDLLDO1
-#endif
-
-#ifdef R_2_5
-#define USE_ENCODING_HINTS
-#endif
-
-#ifdef R_2_6
-// hidden in Rinternals.h if USE_RINTERNALS is defined
-extern Rboolean (Rf_isNull)(SEXP s);
-extern Rboolean (Rf_isObject)(SEXP s);
-SEXP R_LastvalueSymbol;
-// most chars should be const char *, now
-#	define CONSTCHAR const char
-#else
-#	define CONSTCHAR char
-#endif
-
-#ifdef R_2_4
 #include "Rembedded.h"
+
+#ifdef Q_WS_WIN
+#	include "R_ext/RStartup.h"
+#	include "R_ext/Utils.h"
+
+	void RK_scheduleIntr () {
+		UserBreak = 1;
+	}
+
+	void RK_doIntr () {
+		RK_scheduleIntr ();
+		R_CheckUserInterrupt ();
+	}
+
+	structRstart RK_R_Params;
 #else
-extern void R_ReplDLLinit (void);
-extern Rboolean R_ToplevelExec(void (*fun)(void *), void *data);
+#	define RK_doIntr Rf_onintr
+#	include "Rinterface.h"
 #endif
 
 // some functions we need that are not declared
-extern int Rf_initEmbeddedR(int argc, char **argv);
 extern void Rf_PrintWarnings (void);
-extern int R_interrupts_pending;
-#ifdef R_2_3
-extern int Rf_initialize_R(int ac, char **av);
-extern void setup_Rmainloop(void); /* in main.c */
-extern uintptr_t R_CStackLimit;
-extern uintptr_t R_CStackStart;
-extern Rboolean R_Interactive;
+extern int Rf_initialize_R(int ac, char **av);	// in embedded.h in R 2.9.0. TODO: check presence in R 2.7.0
+extern void setup_Rmainloop(void);	// in embedded.h in R 2.9.0. TODO: check presence in R 2.7.0
+#ifndef Q_WS_WIN
+extern uintptr_t R_CStackLimit;	// inRinterface.h in R 2.9.0. TODO: check presence in R 2.7.0
+extern uintptr_t R_CStackStart;	// inRinterface.h in R 2.9.0. TODO: check presence in R 2.7.0
+extern Rboolean R_Interactive;	// inRinterface.h in R 2.9.0. TODO: check presence in R 2.7.0
+#endif
+SEXP R_LastvalueSymbol;
 #include "R_ext/eventloop.h"
-#endif
-#ifndef USE_R_REPLDLLDO1
-extern Rboolean R_Visible;
-#endif
-#ifndef R_2_5
-extern SEXP R_ParseVector(SEXP, int, ParseStatus*);
-#endif
 }
 
 #include "../rkglobals.h"
 #include "rdata.h"
 
-#ifdef USE_R_REPLDLLDO1
+// Needed for the REPL
 const char *current_buffer = 0;
 bool repldlldo1_wants_code = false;
 bool repldll_buffer_transfer_finished = false;
 int repldll_result = 0;		/* -2: error; -1: incomplete; 0: nothing, yet 1: ok 2: incomplete statement, while buffer not empty. Should not happen */
 bool repldll_last_parse_successful = false;
-#endif
+
 
 SEXP RKWard_RData_Tag;
+QString *SEXPToStringList (SEXP from_exp, unsigned int *count);
+int *SEXPToIntArray (SEXP from_exp, unsigned int *count);
 
 // ############## R Standard callback overrides BEGIN ####################
-#ifdef R_2_7
 void RSuicide (const char* message) {
-#else
-void RSuicide (char* message) {
-#endif
 	RK_TRACE (RBACKEND);
 
 	RCallbackArgs args;
-	args.type = RCallbackArgs::RSuicide;
-	args.chars_a = const_cast<char**>(&message);
+	args.type = RCallbackArgs::RBackendExit;
+	args.params["message"] = QVariant (i18n ("The R engine has encountered a fatal error:\n%1").arg (message));
 	REmbedInternal::this_pointer->handleStandardCallback (&args);
 	REmbedInternal::this_pointer->shutdown (true);
 	Rf_error ("Backend dead");	// this jumps us out of the REPL.
 }
 
-#ifdef R_2_7
 void RShowMessage (const char* message) {
-#else
-void RShowMessage (char* message) {
-#endif
 	RK_TRACE (RBACKEND);
 
 	RCallbackArgs args;
 	args.type = RCallbackArgs::RShowMessage;
-	args.chars_a = const_cast<char**>(&message);
+	args.params["message"] = QVariant (message);
 	REmbedInternal::this_pointer->handleStandardCallback (&args);
 }
 
-#ifdef R_2_7
+// TODO: currently used on windows, only!
+/* FROM R_ext/RStartup.h: "Return value here is expected to be 1 for Yes, -1 for No and 0 for Cancel:
+   symbolic constants in graphapp.h" */
+int RAskYesNoCancel (const char* message) {
+	RK_TRACE (RBACKEND);
+
+	RCallbackArgs args;
+	args.type = RCallbackArgs::RShowMessage;
+	args.params["message"] = QVariant (message);
+	args.params["askync"] = QVariant (true);
+
+	REmbedInternal::this_pointer->handleStandardCallback (&args);
+
+	QString ret = args.params["result"].toString ();
+	if (ret == "yes") return 1;
+	if (ret == "no") return -1;
+	return 0;
+}
+
 int RReadConsole (const char* prompt, unsigned char* buf, int buflen, int hist) {
-#else
-int RReadConsole (char* prompt, unsigned char* buf, int buflen, int hist) {
-#endif
 	RK_TRACE (RBACKEND);
 
-#ifdef USE_R_REPLDLLDO1
 	// handle requests for new code
 	if (repldlldo1_wants_code) {
 		//Rprintf ("wants code, buffsize: %d\n", buflen);
@@ -209,64 +191,46 @@ int RReadConsole (char* prompt, unsigned char* buf, int buflen, int hist) {
 		//Rprintf ("buffer now: '%s'\n", buf);
 		return 1;
 	}
-#endif
-	// here, we handle readline calls and such, i.e. not the regular prompt for code
+
+	// here, we handle readline() calls and such, i.e. not the regular prompt for code
+	// browser() also takes us here.
 	RCallbackArgs args;
-	args.type = RCallbackArgs::RReadConsole;
-	args.chars_a = const_cast<char**> (&prompt);
-	args.chars_b = (char **) (&buf);
-	args.int_a = buflen;
-	args.int_b = hist;		// actually, we ignore hist
-	args.int_c = 1;			// not cancelled
+	args.type = RCallbackArgs::RReadLine;
+	args.params["prompt"] = QVariant (prompt);
+	args.params["cancelled"] = QVariant (false);
 
 	REmbedInternal::this_pointer->handleStandardCallback (&args);
 // default implementation seems to return 1 on success, 0 on failure, contrary to some documentation. see unix/std-sys.c
-	if (!(args.int_c)) {
+	if (args.params["cancelled"].toBool ()) {
+#warning TODO: this should be handled in rthread.cpp, instead
 		REmbedInternal::this_pointer->currentCommandWasCancelled ();
-		Rf_onintr ();
+		RK_doIntr();
+		return 0;	// we should not ever get here, but still...
+	}
+	if (buf) {
+		QByteArray localres = REmbedInternal::this_pointer->current_locale_codec->fromUnicode (args.params["result"].toString ());
+		// need to append a newline, here. TODO: theoretically, RReadConsole comes back for more, if \0 was encountered before \n.
+		qstrncpy ((char *) buf, localres.left (buflen - 2).append ('\n').data (), buflen);
+		return 1;
 	}
-	if (buf && args.int_c) return 1;
 	return 0;
 }
 
-#ifdef R_2_5
-#	ifdef R_2_7
+#ifdef Q_WS_WIN
+int RReadConsoleWin (const char* prompt, char* buf, int buflen, int hist) {
+	return RReadConsole (prompt, (unsigned char*) buf, buflen, hist);
+}
+#endif
+
 void RWriteConsoleEx (const char *buf, int buflen, int type) {
-#	else
-void RWriteConsoleEx (char *buf, int buflen, int type) {
-#	endif
 	RK_TRACE (RBACKEND);
 
 	REmbedInternal::this_pointer->handleOutput (REmbedInternal::this_pointer->current_locale_codec->toUnicode (buf, buflen), buflen, type == 0);
 }
-#else
-void RWriteConsole (char *buf, int buflen) {
+
+/** For R callbacks that we want to disable, entirely */
+void RDoNothing () {
 	RK_TRACE (RBACKEND);
-
-	REmbedInternal::this_pointer->handleOutput (REmbedInternal::this_pointer->current_locale_codec->toUnicode (buf, buflen), buflen, true);
-}
-#endif
-
-void RResetConsole () {
-	RK_TRACE (RBACKEND);
-
-// we leave this un-implemented on purpose! We simply don't want that sort of thing to be done.
-}
-
-void RFlushConsole () {
-	RK_TRACE (RBACKEND);
-
-/* nope, we're not going to do the line below after all. Two reasons:
-1) We'd still have to add mutex protection around this call (ok, doable of course)
-2) I don't think we need it at all: We do our own flushing, and R rarely flushes, for obscure reasons, anyway.
-In order to prevent R from doing silly things, we still override this function and leave it unimplemented on purpose. */
-//	REmbedInternal::this_pointer->flushOutput ();
-}
-
-void RClearerrConsole () {
-	RK_TRACE (RBACKEND);
-
-// we leave this un-implemented on purpose! We simply don't want that sort of thing to be done.
 }
 
 void RCleanUp (SA_TYPE saveact, int status, int RunLast) {
@@ -274,9 +238,8 @@ void RCleanUp (SA_TYPE saveact, int status, int RunLast) {
 
 	if (saveact != SA_SUICIDE) {
 		RCallbackArgs args;
-		args.type = RCallbackArgs::RCleanUp;
-		args.int_a = status;
-		args.int_b = RunLast;
+		args.type = RCallbackArgs::RBackendExit;
+		args.params["message"] = QVariant (i18n ("The R engine has shut down with status: %1").arg (status));
 		REmbedInternal::this_pointer->handleStandardCallback (&args);
 
 		if(saveact == SA_DEFAULT) saveact = SA_SAVE;
@@ -294,26 +257,13 @@ void RCleanUp (SA_TYPE saveact, int status, int RunLast) {
 	Rf_error ("Backend dead");	// this jumps us out of the REPL.
 }
 
-#ifdef R_2_7
-int RShowFiles (int nfile, const char **file, const char **headers, const char *wtitle, Rboolean del, const char *pager) {
-#else
-int RShowFiles (int nfile, char **file, char **headers, char *wtitle, Rboolean del, char *pager) {
-#endif
-	RK_TRACE (RBACKEND);
-
-	RCallbackArgs args;
-	args.type = RCallbackArgs::RShowFiles;
-	args.int_a = nfile;
-	args.chars_a = const_cast<char**> (file);
-	args.chars_b = const_cast<char**> (headers);		// what exactly are the "headers"?!?
-	args.chars_c = const_cast<char**> (&wtitle);
-	args.chars_d = const_cast<char**> (&pager);		// we ingnore the pager-parameter for now.
-	args.int_b = del;
-
-	REmbedInternal::this_pointer->handleStandardCallback (&args);
-
-// default implementation seems to returns 1 on success, 0 on failure. see unix/std-sys.c
-	return 1;
+QStringList charPArrayToQStringList (const char** chars, int count) {
+	QStringList ret;
+	for (int i = 0; i < count; ++i) {
+		// do we need to do locale conversion, here?
+		ret.append (chars[i]);
+	}
+	return ret;
 }
 
 int RChooseFile (int isnew, char *buf, int len) {
@@ -321,37 +271,91 @@ int RChooseFile (int isnew, char *buf, int len) {
 
 	RCallbackArgs args;
 	args.type = RCallbackArgs::RChooseFile;
-	args.int_a = isnew;
-	args.int_b = len;
-	args.chars_a = &buf;
+	args.params["new"] = QVariant ((bool) isnew);
 
 	REmbedInternal::this_pointer->handleStandardCallback (&args);
 
+	QByteArray localres = REmbedInternal::this_pointer->current_locale_codec->fromUnicode (args.params["result"].toString ());
+	qstrncpy ((char *) buf, localres.data (), len);
+
 // return length of filename (strlen (buf))
-	return args.int_c;
+	return (qMin (len - 1, localres.size ()));
 }
 
-int REditFiles (int nfile, const char **file, const char **title, const char *editor) {
+/* There are about one million possible entry points to editing / showing files. We try to cover them all, using the
+following bunch of functions (REditFilesHelper() and doShowEditFiles() are helpers, only) */
+
+void REditFilesHelper (QStringList files, QStringList titles, QString wtitle, RCallbackArgs::RCallbackType edit, bool delete_files) {
 	RK_TRACE (RBACKEND);
 
 	RCallbackArgs args;
-	args.type = RCallbackArgs::REditFiles;
-	args.int_a = nfile;
-	args.chars_a = const_cast<char**> (file);
-	args.chars_b = const_cast<char**> (title);
-	args.chars_c = const_cast<char**> (&editor);
+	if (edit == RCallbackArgs::REditFiles) args.type = RCallbackArgs::REditFiles;
+	else {
+		RK_ASSERT (edit == RCallbackArgs::RShowFiles);
+		args.type = RCallbackArgs::RShowFiles;
+		args.params["delete"] = QVariant (delete_files);
+	}
+	// see ?file.show() for what appears to be the intended meaning of these first three parameters
+	// (which seem to be inconsistently named even in R itself...)
+	args.params["files"] = QVariant (files);
+	args.params["titles"] = QVariant (titles);
+	args.params["wtitle"] = QVariant (wtitle);
 
 	REmbedInternal::this_pointer->handleStandardCallback (&args);
+}
+
+int REditFiles (int nfile, const char **file, const char **title, const char *wtitle) {
+	RK_TRACE (RBACKEND);
+
+	REditFilesHelper (charPArrayToQStringList (file, nfile), charPArrayToQStringList (title, nfile), wtitle, RCallbackArgs::REditFiles, false);
 
 // default implementation seems to return 1 if nfile <= 0, else 1. No idea, what for. see unix/std-sys.c
 	return (nfile <= 0);
 }
 
-#ifdef R_2_7
+SEXP doShowEditFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP del, RCallbackArgs::RCallbackType edit) {
+	RK_TRACE (RBACKEND);
+
+	// this function would be much shorter, if SEXPToStringList would simply return a QStringList...
+	unsigned int files_count, titles_count, wtitle_count, del_count;
+	QString *file_strings = SEXPToStringList (files, &files_count);
+	QString *title_strings = SEXPToStringList (titles, &titles_count);
+	QString *wtitle_strings = SEXPToStringList (wtitle, &wtitle_count);
+	int *del_bools = SEXPToIntArray (del, &del_count);
+
+	RK_ASSERT (wtitle_count <= 1);
+	RK_ASSERT (del_count <= 1);
+	RK_ASSERT (files_count == titles_count);
+	RK_ASSERT (files_count >= 1);
+
+	files_count = titles_count = qMin (files_count, titles_count);
+
+	QStringList files_list;
+	QStringList titles_list;
+	for (unsigned int i = 0; i < files_count; ++i) {
+		files_list.append (file_strings[i]);
+		titles_list.append (title_strings[i]);
+	}
+	QString wtitle_string;
+	if (wtitle_count) wtitle_string = wtitle_strings[0];
+	bool del_files = false;
+	if (del_count) del_files = (del_bools[0] != 0);
+
+	REditFilesHelper (files_list, titles_list, wtitle_string, edit, del_files);
+
+	delete [] file_strings;
+	delete [] title_strings;
+	delete [] wtitle_strings;
+	delete [] del_bools;
+
+	return (R_NilValue);
+}
+
+SEXP doEditFiles (SEXP files, SEXP titles, SEXP wtitle) {
+	return (doShowEditFiles (files, titles, wtitle, R_NilValue, RCallbackArgs::REditFiles));
+}
+
 int REditFile (const char *buf) {
-#else
-int REditFile (char *buf) {
-#endif
 	RK_TRACE (RBACKEND);
 
 	const char *editor = "none";
@@ -361,7 +365,19 @@ int REditFile (char *buf) {
 	return REditFiles (1, const_cast<const char**> (&buf), &title, editor);
 }
 
-#ifdef USE_R_REPLDLLDO1
+SEXP doShowFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP delete_files) {
+	return (doShowEditFiles (files, titles, wtitle, delete_files, RCallbackArgs::RShowFiles));
+}
+
+int RShowFiles (int nfile, const char **file, const char **headers, const char *wtitle, Rboolean del, const char */* pager */) {
+	RK_TRACE (RBACKEND);
+
+	REditFilesHelper (charPArrayToQStringList (file, nfile), charPArrayToQStringList (headers, nfile), QString (wtitle), RCallbackArgs::RShowFiles, (bool) del);
+
+// default implementation seems to returns 1 on success, 0 on failure. see unix/std-sys.c
+	return 1;
+}
+
 void RBusy (int busy) {
 	RK_TRACE (RBACKEND);
 
@@ -371,7 +387,6 @@ void RBusy (int busy) {
 		repldll_last_parse_successful = true;
 	}
 }
-#endif
 
 // ############## R Standard callback overrides END ####################
 
@@ -384,37 +399,61 @@ REmbedInternal::REmbedInternal () {
 	r_running = false;
 }
 
+#ifdef Q_WS_WIN
+void REmbedInternal::setupCallbacks () {
+	RK_TRACE (RBACKEND);
+
+	R_setStartTime();
+	R_DefParams(&RK_R_Params);
+
+// IMPORTANT: see also the #ifndef QS_WS_WIN-portion!
+	RK_R_Params.rhome = get_R_HOME ();
+	RK_R_Params.home = getRUser ();
+	RK_R_Params.CharacterMode = LinkDLL;
+	RK_R_Params.ShowMessage = RShowMessage;
+	RK_R_Params.ReadConsole = RReadConsoleWin;
+	RK_R_Params.WriteConsoleEx = RWriteConsoleEx;
+	RK_R_Params.WriteConsole = 0;
+	RK_R_Params.CallBack = RDoNothing;
+	RK_R_Params.YesNoCancel = RAskYesNoCancel;
+	RK_R_Params.Busy = RBusy;
+
+	// TODO: callback mechanism(s) for ChosseFile, ShowFiles, EditFiles
+	// TODO: also for RSuicide / RCleanup? (Less important, obviously, since those should not be triggered, in normal operation).
+
+	RK_R_Params.R_Quiet = (Rboolean) 0;
+	RK_R_Params.R_Interactive = (Rboolean) 1;
+}
+
+void REmbedInternal::connectCallbacks () {
+	RK_TRACE (RBACKEND);
+	R_SetParams(&RK_R_Params);
+}
+#else
+void REmbedInternal::setupCallbacks () {
+	RK_TRACE (RBACKEND);
+}
+
 void REmbedInternal::connectCallbacks () {
 	RK_TRACE (RBACKEND);
 
-// R standard callback pointers.
-// Rinterface.h thinks this can only ever be done on aqua, apparently. Here, we define it the other way around, i.e. #ifndef instead of #ifdef
-// No, does not work -> undefined reference! -> TODO: nag R-devels
-#ifndef HAVE_AQUA
-	//extern int  (*ptr_R_EditFiles)(int, char **, char **, char *);
-#endif
-
+// IMPORTANT: see also the #ifdef QS_WS_WIN-portion!
 // connect R standard callback to our own functions. Important: Don't do so, before our own versions are ready to be used!
 	R_Outputfile = NULL;
 	R_Consolefile = NULL;
 	ptr_R_Suicide = RSuicide;
-	ptr_R_ShowMessage = RShowMessage;			// when exactly does this ever get used?
+	ptr_R_ShowMessage = RShowMessage;		// rarely used in R on unix
 	ptr_R_ReadConsole = RReadConsole;
-#ifdef R_2_5
 	ptr_R_WriteConsoleEx = RWriteConsoleEx;
 	ptr_R_WriteConsole = 0;
-#else
-	ptr_R_WriteConsole = RWriteConsole;
-#endif
-	ptr_R_ResetConsole = RResetConsole;
-	ptr_R_FlushConsole = RFlushConsole;
-	ptr_R_ClearerrConsole = RClearerrConsole;
-#ifdef USE_R_REPLDLLDO1
+	ptr_R_ResetConsole = RDoNothing;
+	ptr_R_FlushConsole = RDoNothing;
+	ptr_R_ClearerrConsole = RDoNothing;
 	ptr_R_Busy = RBusy;
-#endif
 	ptr_R_CleanUp = RCleanUp;			// unfortunately, it seems, we can't safely cancel quitting anymore, here!
 	ptr_R_ShowFiles = RShowFiles;
 	ptr_R_ChooseFile = RChooseFile;
+// TODO: R devels disabled this for some reason. We set it anyway...
 	ptr_R_EditFile = REditFile;
 //	ptr_R_EditFiles = REditFiles;		// undefined reference
 
@@ -422,6 +461,7 @@ void REmbedInternal::connectCallbacks () {
 //	ptr_R_loadhistory = ... 	// we keep our own history
 //	ptr_R_savehistory = ...	// we keep our own history
 }
+#endif
 
 REmbedInternal::~REmbedInternal () {
 	RK_TRACE (RBACKEND);
@@ -433,55 +473,31 @@ void REmbedInternal::shutdown (bool suicidal) {
 	if (!r_running) return;		// already shut down
 	r_running = false;
 
-// Code-recipe below essentially copied from http://stat.ethz.ch/R-manual/R-devel/doc/manual/R-exts.html#Linking-GUIs-and-other-front_ends-to-R
-// modified quite a bit for our needs.
-
-	if (!suicidal) {
-		R_dot_Last ();
-	}
-
-	R_RunExitFinalizers();
-
-#ifdef R_2_4
-	R_CleanTempDir ();
-#else
-	char *tmpdir;
-	if((tmpdir = getenv("R_SESSION_TMPDIR"))) {
-		char buf[1024];
-		snprintf((char *)buf, 1024, "rm -rf %s", tmpdir);
-		R_system((char *)buf);
-	}
-#endif
-
-	/* close all the graphics devices */
-	if (!suicidal) Rf_KillAllDevices ();
-	fpu_setup ((Rboolean) FALSE);
+	Rf_endEmbeddedR (suicidal);
 }
 
+#if 0
 static int timeout_counter = 0;
+#endif
 
 void processX11EventsWorker (void *) {
-#ifdef R_2_3
 // this basically copied from R's unix/sys-std.c (Rstd_ReadConsole)
-// we stop processing, if there are more than 10 events
+#ifndef Q_WS_WIN
 	for (;;) {
 		fd_set *what;
-		what = R_checkActivityEx(R_wait_usec > 0 ? R_wait_usec : 50, 1, Rf_onintr);
+		what = R_checkActivityEx(R_wait_usec > 0 ? R_wait_usec : 50, 1, RK_doIntr);
 		if (what == NULL) break;
 		R_runHandlers(R_InputHandlers, what);
 	}
+	/* This seems to be needed to make Rcmdr react to events. Has this always been the case? It was commented out for a long time, without anybody noticing. */
+	R_PolledEvents ();
 #else
-/* what we do here is walk the list of objects, that have told R, they're listening for events.
-We figure out which ones look for X11-events and tell those to do their stuff (regardless of whether events actually occurred) */
-	extern InputHandler *R_InputHandlers;
-	InputHandler *handler = R_InputHandlers;
-	while (handler) {
-		if (handler->activity == XActivity) {
-			handler->handler ((void*) 0);
-		}
-		handler = handler->next;
-	}
+#warning TODO: correct?
+	R_ProcessEvents();
+#endif
 
+#if 0
+// TODO: The remainder of this function had been commented out since R 2.3.x and is not in Rstd_ReadConsole. Do we still need this?
 	/* I don't really understand what I'm doing here, but apparently this is necessary for Tcl-Tk windows to function properly. */
 	R_PolledEvents ();
 	
@@ -520,7 +536,11 @@ QString *SEXPToStringList (SEXP from_exp, unsigned int *count) {
 	QString *list = new QString[*count];
 	unsigned int i = 0;
 	for (; i < *count; ++i) {
+#ifdef R_2_9
+		SEXP dummy = STRING_ELT (from_exp, i);
+#else
 		SEXP dummy = VECTOR_ELT (from_exp, i);
+#endif
 
 		if (TYPEOF (dummy) != CHARSXP) {
 			list[i] = QString ("not defined");	// can this ever happen?
@@ -528,7 +548,6 @@ QString *SEXPToStringList (SEXP from_exp, unsigned int *count) {
 			if (dummy == NA_STRING) {
 				list[i] = QString::null;
 			} else {
-#ifdef USE_ENCODING_HINTS
 				if (IS_UTF8 (dummy)) {
 					list[i] = QString::fromUtf8 ((char *) STRING_PTR (dummy));
 				} else if (IS_LATIN1 (dummy)) {
@@ -536,9 +555,6 @@ QString *SEXPToStringList (SEXP from_exp, unsigned int *count) {
 				} else {
 					list[i] = REmbedInternal::this_pointer->current_locale_codec->toUnicode ((char *) STRING_PTR (dummy));
 				}
-#else
-				list[i] = REmbedInternal::this_pointer->current_locale_codec->toUnicode ((char *) STRING_PTR (dummy));
-#endif
 			}
 		}
 	}
@@ -673,69 +689,18 @@ SEXP doSubstackCall (SEXP call) {
 
 	unsigned int count;
 	QString *strings = SEXPToStringList (call, &count);
-	REmbedInternal::this_pointer->handleSubstackCall (strings, count);
+	QStringList list;
+	for (unsigned int i = 0; i < count; ++i) {
+		list.append (strings[i]);
+	}
+	REmbedInternal::this_pointer->handleSubstackCall (list);
 	delete [] strings;
 	return R_NilValue;
 }
 
-#ifdef R_2_3
 void R_CheckStackWrapper (void *) {
 	R_CheckStack ();
 }
-#endif
-
-bool REmbedInternal::startR (int argc, char** argv, bool stack_check) {
-	RK_TRACE (RBACKEND);
-
-	RKSignalSupport::saveDefaultSigSegvHandler ();
-
-	r_running = true;
-	bool ok = true;
-#ifdef R_2_3
-	Rf_initialize_R (argc, argv);
-
-	if (stack_check) {
-		char dummy;
-		size_t stacksize;
-		void *stackstart;
-		RKGetCurrentThreadStackLimits (&stacksize, &stackstart, &dummy);
-		R_CStackStart = (uintptr_t) stackstart;
-		R_CStackLimit = stacksize;
-	} else {
-		R_CStackStart = (uintptr_t) -1;
-		R_CStackLimit = (uintptr_t) -1;
-	}
-
-	setup_Rmainloop ();
-
-	if (stack_check) {
-		// safety check: If we are beyond the stack boundaries already, we better disable stack checking
-		// this has to come *after* the first setup_Rmainloop ()!
-		Rboolean stack_ok = R_ToplevelExec (R_CheckStackWrapper, (void *) 0);
-		if (!stack_ok) {
-			RK_DO (qDebug ("R_CheckStack() failed during initialization. Will disable stack checking and try to re-initialize."), RBACKEND, DL_WARNING);
-			RK_DO (qDebug ("If this does not work, try the --disable-stack-check command line option, *and* submit a bug report."), RBACKEND, DL_WARNING);
-			R_CStackStart = (uintptr_t) -1;
-			R_CStackLimit = (uintptr_t) -1;
-			setup_Rmainloop ();
-		}
-	}
-
-	R_Interactive = (Rboolean) TRUE;
-#else
-	ok = (Rf_initEmbeddedR (argc, argv) >= 0);
-#endif
-	RKGlobals::na_double = NA_REAL;
-	R_ReplDLLinit ();
-	RKWard_RData_Tag = Rf_install ("RKWard_RData_Tag");
-#ifdef R_2_6
-	R_LastvalueSymbol = Rf_install (".Last.value");
-#endif
-
-	RKSignalSupport::installSigSegvProxy ();
-
-	return ok;
-}
 
 SEXP doUpdateLocale () {
 	RK_TRACE (RBACKEND);
@@ -757,6 +722,12 @@ SEXP doGetStructure (SEXP toplevel, SEXP name, SEXP envlevel, SEXP namespacename
 	return R_MakeExternalPtr (ret, RKWard_RData_Tag, R_NilValue);
 }
 
+SEXP doGetGlobalEnvStructure (SEXP name, SEXP envlevel, SEXP namespacename) {
+	RK_TRACE (RBACKEND);
+
+	return doGetStructure (findVar (Rf_install (CHAR (STRING_ELT (name, 0))), R_GlobalEnv), name, envlevel, namespacename);
+}
+
 /** copy a symbol without touching it (esp. not forcing any promises) */
 SEXP doCopyNoEval (SEXP name, SEXP fromenv, SEXP toenv) {
 	RK_TRACE (RBACKEND);
@@ -768,22 +739,81 @@ SEXP doCopyNoEval (SEXP name, SEXP fromenv, SEXP toenv) {
 	return (R_NilValue);
 }
 
-bool REmbedInternal::registerFunctions (const char *library_path) {
+bool REmbedInternal::startR (int argc, char** argv, bool stack_check) {
 	RK_TRACE (RBACKEND);
 
-	DllInfo *info = R_getDllInfo (library_path);
-	if (!info) return false;
+	setupCallbacks ();
 
+	RKSignalSupport::saveDefaultSigSegvHandler ();
+
+	r_running = true;
+	Rf_initialize_R (argc, argv);
+
+#ifndef Q_WS_WIN
+	// in R on windows the stack limits detection seems to work out of the box for threads
+	if (stack_check) {
+		char dummy;
+		size_t stacksize;
+		void *stackstart;
+		RKGetCurrentThreadStackLimits (&stacksize, &stackstart, &dummy);
+		R_CStackStart = (uintptr_t) stackstart;
+		R_CStackLimit = stacksize;
+	} else {
+		R_CStackStart = (uintptr_t) -1;
+		R_CStackLimit = (uintptr_t) -1;
+	}
+#endif
+
+#ifdef Q_WS_WIN
+	R_set_command_line_arguments(argc, argv);
+	FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE));
+#endif
+
+	setup_Rmainloop ();
+
+#ifndef Q_WS_WIN
+	// in R on windows the stack limits detection seems to work out of the box for threads
+	if (stack_check) {
+		// safety check: If we are beyond the stack boundaries already, we better disable stack checking
+		// this has to come *after* the first setup_Rmainloop ()!
+		Rboolean stack_ok = R_ToplevelExec (R_CheckStackWrapper, (void *) 0);
+		if (!stack_ok) {
+			RK_DO (qDebug ("R_CheckStack() failed during initialization. Will disable stack checking and try to re-initialize."), RBACKEND, DL_WARNING);
+			RK_DO (qDebug ("If this does not work, try the --disable-stack-check command line option, *and* submit a bug report."), RBACKEND, DL_WARNING);
+			R_CStackStart = (uintptr_t) -1;
+			R_CStackLimit = (uintptr_t) -1;
+			setup_Rmainloop ();
+		}
+	}
+#endif
+
+#ifndef Q_WS_WIN
+	// on windows, set in connectCallbacks() for technical reasons
+	R_Interactive = (Rboolean) TRUE;
+#endif
+
+	RKGlobals::na_double = NA_REAL;
+	R_ReplDLLinit ();
+
+	RKWard_RData_Tag = Rf_install ("RKWard_RData_Tag");
+	R_LastvalueSymbol = Rf_install (".Last.value");
+
+	RKSignalSupport::installSigSegvProxy ();
+
+// register our functions
 	R_CallMethodDef callMethods [] = {
 //		{ "rk.do.condition", (DL_FUNC) &doCondition, 1 },
 		{ "rk.do.error", (DL_FUNC) &doError, 1 },
 		{ "rk.do.command", (DL_FUNC) &doSubstackCall, 1 },
 		{ "rk.update.locale", (DL_FUNC) &doUpdateLocale, 0 },
 		{ "rk.get.structure", (DL_FUNC) &doGetStructure, 4 },
+		{ "rk.get.structure.global", (DL_FUNC) &doGetGlobalEnvStructure, 3 },
 		{ "rk.copy.no.eval", (DL_FUNC) &doCopyNoEval, 3 },
+		{ "rk.edit.files", (DL_FUNC) &doEditFiles, 3 },
+		{ "rk.show.files", (DL_FUNC) &doShowFiles, 4 },
 		{ 0, 0, 0 }
 	};
-	R_registerRoutines (info, NULL, callMethods, NULL, NULL);
+	R_registerRoutines (R_getEmbeddingDllInfo(), NULL, callMethods, NULL, NULL);
 
 	return true;
 }
@@ -798,14 +828,14 @@ SEXP parseCommand (const QString &command_qstring, REmbedInternal::RKWardRError
 	const char *command = localc.data ();
 
 	PROTECT(cv=allocVector(STRSXP, 1));
-	SET_VECTOR_ELT(cv, 0, mkChar(command));  
+#ifdef R_2_9
+	SET_STRING_ELT(cv, 0, mkChar(command));
+#else
+	SET_VECTOR_ELT(cv, 0, mkChar(command));
+#endif
 
 	// TODO: Maybe we can use R_ParseGeneral instead. Then we could find the exact character, where parsing fails. Nope: not exported API
-#ifdef R_2_5
 	pr=R_ParseVector(cv, -1, &status, R_NilValue);
-#else
-	pr=R_ParseVector(cv, -1, &status);
-#endif
 	UNPROTECT(1);
 
 	if ((!pr) || (TYPEOF (pr) == NILSXP)) {
@@ -876,7 +906,8 @@ SEXP runCommandInternalBase (SEXP pr, REmbedInternal::RKWardRError *error) {
 	return exp;
 }
 
-#ifndef USE_R_REPLDLLDO1
+#if 0
+// This is currently unused, but might come in handy, again.
 /* Basically a safe version of Rf_PrintValue, as yes, Rf_PrintValue may lead to an error and long_jump->crash!
 For example in help (function, htmlhelp=TRUE), when no HTML-help is installed!
 SEXP exp should be PROTECTed prior to calling this function.
@@ -907,7 +938,6 @@ void tryPrintValue (SEXP exp, REmbedInternal::RKWardRError *error) {
 }
 #endif
 
-#ifdef USE_R_REPLDLLDO1
 void runUserCommandInternal (void *) {
 	RK_TRACE (RBACKEND);
 
@@ -924,7 +954,6 @@ void runUserCommandInternal (void *) {
 	//Rprintf ("iteration complete, status: %d\n", repldll_result);
 	PROTECT (R_LastvalueSymbol);		// why do we need this? No idea, but R_ToplevelExec tries to unprotect something
 }
-#endif
 
 void REmbedInternal::runCommandInternal (const QString &command_qstring, RKWardRError *error, bool print_result) {
 	RK_TRACE (RBACKEND);
@@ -936,14 +965,6 @@ void REmbedInternal::runCommandInternal (const QString &command_qstring, RKWardR
 		SEXP parsed = parseCommand (command_qstring, error);
 		if (*error == NoError) runCommandInternalBase (parsed, error);
 	} else {		// run a user command
-#ifndef USE_R_REPLDLLDO1
-		SEXP parsed = parseCommand (command_qstring, error);
-#endif
-		// do not run incomplete commands, but *do* run commands with syntax errors if USE_R_REPLDLLDO1. Why? Because this is the only way to get a syntax error messages, so far.
-		if ((*error != NoError)) {
-			if (*error != SyntaxError) return;
-		}
-#ifdef USE_R_REPLDLLDO1
 /* Using R_ReplDLLdo1 () is a pain, but it seems to be the only entry point for evaluating a command as if it had been entered on a plain R console (with auto-printing if not invisible, etc.). Esp. since R_Visible is no longer exported in R 2.5.0, as it seems as of today (2007-01-17).
 
 Problems to deal with:
@@ -996,29 +1017,6 @@ hist == 1 iff R wants a parse-able input.
 			}
 		}
 		repldlldo1_wants_code = false;		// make sure we don't get confused in RReadConsole
-
-#else
-		// in the non USE_R_REPLDLLDO1 case, any error is fatal, and we should not try running the command at all.
-		if ((*error) != NoError) return;
-
-		R_Visible = (Rboolean) 0;
-
-		SEXP exp;
-		PROTECT (exp = runCommandInternalBase (parsed, error));
-
-		if (*error == NoError) {
-			SET_SYMVALUE (R_LastvalueSymbol, exp);
-			if (R_Visible) {
-				tryPrintValue (exp, error);
-			}
-		}
-		UNPROTECT (1); /* exp */
-
-		/* See the comment in the corresponding code in runCommandInternalBase. And yes, apparently, we need this at both places! */
-		Rprintf ("");
-
-		Rf_PrintWarnings ();
-#endif
 	}
 }
 
