@@ -2,7 +2,7 @@
                           rinterface.cpp  -  description
                              -------------------
     begin                : Fri Nov 1 2002
-    copyright            : (C) 2002, 2004, 2005, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2002, 2004, 2005, 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -32,6 +32,8 @@
 #include "../windows/rcontrolwindow.h"
 #include "../windows/rkworkplace.h"
 #include "../windows/rkcommandlog.h"
+#include "../plugin/rkcomponentmap.h"
+#include "../misc/rkcommonfunctions.h"
 
 #include "../windows/rkwindowcatcher.h"
 
@@ -77,6 +79,7 @@ RInterface::RInterface () {
 	new RCommandStackModel (this);
 	RCommandStack::regular_stack = new RCommandStack (0);
 	running_command_canceled = 0;
+	command_logfile_mode = NotRecordingCommands;
 
 	r_thread = new RThread ();
 
@@ -166,7 +169,15 @@ void RInterface::customEvent (QEvent *e) {
 			r_thread->pauseOutput (false);
 		}
 	} else if (ev->etype () == RKRBackendEvent::RCommandIn) {
-		RKCommandLog::getLog ()->addInput (static_cast <RCommand *> (ev->data ()));
+		RCommand *command = static_cast <RCommand *> (ev->data ());
+		RKCommandLog::getLog ()->addInput (command);
+
+		if (command_logfile_mode != NotRecordingCommands) {
+			if ((!(command->type () & RCommand::Sync)) || command_logfile_mode == RecordingCommandsWithSync) {
+				command_logfile.write (command->command ().toUtf8 ());
+				command_logfile.write ("\n");
+			}
+		}
 	} else if (ev->etype () == RKRBackendEvent::RCommandOut) {
 		RCommand *command = static_cast <RCommand *> (ev->data ());
 		if (command->status & RCommand::Canceled) {
@@ -184,11 +195,6 @@ void RInterface::customEvent (QEvent *e) {
 			}
 		}
 		command->finished ();
-		if (command->type () & RCommand::DirectToOutput) {
-			RKWorkplace::mainWorkplace ()->refreshOutputWindow ();
-		} else if (command->type () & RCommand::User) {
-			RKWorkplace::mainWorkplace ()->refreshOutputWindow ();
-		}
 		delete command;
 	} else if ((ev->etype () == RKRBackendEvent::RIdle)) {
 		RKWardMainWindow::getMain ()->setRStatus (RKWardMainWindow::Idle);	
@@ -207,18 +213,24 @@ void RInterface::customEvent (QEvent *e) {
 		int* err_p = static_cast<int*> (ev->data ());
 		int err = *err_p;
 		delete err_p;
-		QString message = i18n ("There was a problem starting the R backend. The following error(s) occurred:\n");
+		QString message = i18n ("<p>There was a problem starting the R backend. The following error(s) occurred:</p>\n");
 		if (err & RThread::LibLoadFail) {
-			message.append (i18n ("\t- The 'rkward' R-library could not be loaded. This library is needed for communication between R and RKWard and many things will not work properly if this library is not present. Likely RKWard will even crash. The 'rkward' R-library should have been included in your distribution or RKWard, and should have been set up when you ran 'make install'. Please try 'make install' again and check for any errors. You should quit RKWard now.\n"));
+			message.append (i18n ("</p>\t- The 'rkward' R-library either could not be loaded at all, or not in the correct version. This may lead to all sorts of errors, from single missing features to complete failure to function. The most likely cause is that the last installation did not place all files in the correct place. However, in some cases, left-overs from a previous installation that was not cleanly removed may be the cause.</p>\
+			<p><b>You should quit RKWard, now, and fix your installation</b>. For help with that, see <a href=\"http://p.sf.net/rkward/compiling\">http://p.sf.net/rkward/compiling</a>.</p>\n"));
 		}
 		if (err & RThread::SinkFail) {
-			message.append (i18n ("\t-There was a problem opening the files needed for communication with R. Most likely this is due to an incorrect setting for the location of these files. Check whether you have correctly configured the location of the log-files (Settings->Configure Settings->Logfiles) and restart RKWard.\n"));
+			message.append (i18n ("<p>\t-There was a problem opening the files needed for communication with R. Most likely this is due to an incorrect setting for the location of these files. Check whether you have correctly configured the location of the log-files (Settings->Configure Settings->Logfiles) and restart RKWard.</p>\n"));
 		}
 		if (err & RThread::OtherFail) {
-			message.append (i18n ("\t-An unspecified error occurred that is not yet handled by RKWard. Likely RKWard will not function properly. Please check your setup.\n"));
+			message.append (i18n ("<p>\t-An unspecified error occurred that is not yet handled by RKWard. Likely RKWard will not function properly. Please check your setup.</p>\n"));
 		}
-		KMessageBox::error (0, message, i18n ("Error starting R"));
-		r_thread->unlock (RThread::Startup);
+		QString details = runningCommand()->fullOutput();
+		if (!details.isEmpty ()) {
+			// WORKAROUND for stupid KMessageBox behavior. (kdelibs 4.2.3)
+			// If length of details <= 512, it tries to show the details as a QLabel.
+			details = details.replace('<', "&lt;").replace('\n', "<br>").leftJustified (513);
+		}
+		KMessageBox::detailedError (0, message, details, i18n ("Error starting R"), KMessageBox::Notify | KMessageBox::AllowLink);
 	} else {
 		RK_ASSERT (false);
 	}
@@ -291,7 +303,7 @@ void RInterface::processREvalRequest (REvalRequest *request) {
 
 	// clear reply object
 	issueCommand (".rk.set.reply (NULL)", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
-	if (!request->call_length) {
+	if (request->call.isEmpty ()) {
 		RK_ASSERT (false);
 		closeChain (request->in_chain);
 		return;
@@ -299,7 +311,7 @@ void RInterface::processREvalRequest (REvalRequest *request) {
 	
 	QString call = request->call[0];
 	if (call == "get.tempfile.name") {
-		if (request->call_length >= 3) {
+		if (request->call.count () >= 3) {
 			QString file_prefix = request->call[1];
 			QString file_extension = request->call[2];
 			QDir dir (RKSettingsModuleGeneral::filesPath ());
@@ -312,10 +324,14 @@ void RInterface::processREvalRequest (REvalRequest *request) {
 		} else {
 			issueCommand (".rk.set.reply (\"Too few arguments in call to get.tempfile.name.\")", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
 		}
-	} else if (call == "sync") {
-		RK_ASSERT (request->call_length >= 2);
+	} else if (call == "refreshOutput") {
+		RK_ASSERT (request->call.count () == 2); //but we don't use the second parameter, yet
 
-		for (int i = 1; i < request->call_length; ++i) {
+		RKWorkplace::mainWorkplace ()->refreshOutputWindow ();
+	} else if (call == "sync") {
+		RK_ASSERT (request->call.count () >= 2);
+
+		for (int i = 1; i < request->call.count (); ++i) {
 			QString object_name = request->call[i];
 			RObject *obj = RObjectList::getObjectList ()->findObject (object_name);
 			if (obj) {
@@ -326,26 +342,19 @@ void RInterface::processREvalRequest (REvalRequest *request) {
 				RK_DO (qDebug ("lookup failed for changed symbol %s", object_name.toLatin1 ().data()), RBACKEND, DL_WARNING);
 			}
 		}
-	} else if (call == "syncall") {
-		RK_ASSERT (request->call_length == 1);
-
+	} else if (call == "syncenvs") {
 		RK_DO (qDebug ("triggering update of object list"), RBACKEND, DL_DEBUG);
-		RObjectList::getObjectList ()->updateFromR (request->in_chain);
+		RObjectList::getObjectList ()->updateFromR (request->in_chain, request->call.mid (1));
 	} else if (call == "syncglobal") {
-		RK_ASSERT (request->call_length == 1);
-
 		RK_DO (qDebug ("triggering update of globalenv"), RBACKEND, DL_DEBUG);
-		RObjectList::getGlobalEnv ()->updateFromR (request->in_chain);
+		RObjectList::getGlobalEnv ()->updateFromR (request->in_chain, request->call.mid (1));
 	} else if (call == "edit") {
-		RK_ASSERT (request->call_length >= 2);
+		RK_ASSERT (request->call.count () >= 2);
 
-		QStringList object_list;
-		for (int i = 1; i < request->call_length; ++i) {
-			object_list.append (request->call[i]);
-		}
+		QStringList object_list = request->call.mid (1);
 		new RKEditObjectAgent (object_list, request->in_chain);
 	} else if (call == "require") {
-		if (request->call_length >= 2) {
+		if (request->call.count () >= 2) {
 			QString lib_name = request->call[1];
 			KMessageBox::information (0, i18n ("The R-backend has indicated that in order to carry out the current task it needs the package '%1', which is not currently installed. We will open the package-management tool, and there you can try to locate and install the needed package.", lib_name), i18n ("Require package '%1'", lib_name));
 			RKLoadLibsDialog::showInstallPackagesModal (0, request->in_chain, lib_name);
@@ -360,14 +369,14 @@ void RInterface::processREvalRequest (REvalRequest *request) {
 #ifndef DISABLE_RKWINDOWCATCHER
  	} else if (call == "startOpenX11") {
 		// TODO: error checking/handling (wrong parameter count/type)
-		if (request->call_length >= 2) {
+		if (request->call.count () >= 2) {
 			MUTEX_LOCK;
 			window_catcher->start (QString (request->call[1]).toInt ());
 			MUTEX_UNLOCK;
 		}
  	} else if (call == "endOpenX11") {
 		// TODO: error checking/handling (wrong parameter count/type)
-		if (request->call_length >= 2) {
+		if (request->call.count () >= 2) {
 			MUTEX_LOCK;
 			window_catcher->stop (QString (request->call[1]).toInt ());
 			MUTEX_UNLOCK;
@@ -380,6 +389,56 @@ void RInterface::processREvalRequest (REvalRequest *request) {
 		if (res != KMessageBox::Continue) {
 			issueCommand (".rk.set.reply (FALSE)", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
 		}
+	} else if (call == "doPlugin") {
+		if (request->call.count () >= 3) {
+			QString message;
+			bool ok;
+			RKComponentMap::ComponentInvocationMode mode = RKComponentMap::ManualSubmit;
+			if (request->call[2] == "auto") mode = RKComponentMap::AutoSubmit;
+			else if (request->call[2] == "submit") mode = RKComponentMap::AutoSubmitOrFail;
+			ok = RKComponentMap::invokeComponent (request->call[1], request->call.mid (3), mode, &message, request->in_chain);
+
+			if (message.isEmpty ()) {
+				issueCommand (".rk.set.reply (NULL)", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
+			} else {
+				QString type = "warning";
+				if (!ok) type = "error";
+				issueCommand (".rk.set.reply (list (type=\"" + type + "\", message=\"" + RKCommonFunctions::escape (message) + "\"))", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
+			}
+		} else {
+			RK_ASSERT (false);
+		}
+	} else if (call == "showHTML") {
+		if (request->call.count () == 2) {
+			RKWorkplace::mainWorkplace ()->openHelpWindow (request->call[1]);
+		} else {
+			RK_ASSERT (false);
+		}
+	} else if (call == "recordCommands") {
+		if (request->call.count () == 3) {
+			QString filename = request->call[1];
+			bool with_sync = (request->call[2] == "include.sync");
+
+			if (filename.isEmpty ()) {
+				command_logfile_mode = NotRecordingCommands;
+				command_logfile.close ();
+			} else {
+				if (command_logfile_mode != NotRecordingCommands) {
+					issueCommand (".rk.set.reply (\"Attempt to start recording, while already recording commands. Ignoring.\")", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
+				} else {
+					command_logfile.setFileName (filename);
+					bool ok = command_logfile.open (QIODevice::WriteOnly | QIODevice::Truncate);
+					if (ok) {
+						command_logfile_mode = RecordingCommands;
+						if (with_sync) command_logfile_mode = RecordingCommandsWithSync;
+					} else {
+						issueCommand (".rk.set.reply (\"Could not open file for writing. Not recording commands.\")", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
+					}
+				}
+			}
+		} else {
+			RK_ASSERT (false);
+		}
 	} else {
 		issueCommand (".rk.set.reply (\"Unrecognized call '" + call + "'. Ignoring\")", RCommand::App | RCommand::Sync, QString::null, 0, 0, request->in_chain);
 	}
@@ -394,45 +453,35 @@ void RInterface::processRCallbackRequest (RCallbackArgs *args) {
 	RCallbackArgs::RCallbackType type = args->type;
 
 	if (type == RCallbackArgs::RShowMessage) {
-		KMessageBox::information (0, QString (*(args->chars_a)), i18n ("Message from the R backend"));
-	} else if (type == RCallbackArgs::RReadConsole) {
+		if (args->params.value ("askync").toBool ()) {
+			int res = KMessageBox::questionYesNoCancel (0, args->params["message"].toString (), i18n ("Question from the R backend"));
+			if (res == KMessageBox::Yes) args->params["result"] = "yes";
+			else if (res == KMessageBox::No) args->params["result"] = "no";
+			// else: cancel
+		} else {
+			KMessageBox::information (0, args->params["message"].toString (), i18n ("Message from the R backend"));
+		}
+	} else if (type == RCallbackArgs::RReadLine) {
 		QString result;
 
-		bool ok = RKReadLineDialog::readLine (0, i18n ("R backend requests information"), *(args->chars_a), runningCommand (), &result);
+		bool ok = RKReadLineDialog::readLine (0, i18n ("R backend requests information"), args->params["prompt"].toString (), runningCommand (), &result);
+		args->params["result"] = QVariant (result);
 
-		result = result.left (args->int_a - 2) + '\n';
-		qstrcpy (*(args->chars_b), result.toLocal8Bit ());
-
-		if (!ok) args->int_c = 0;	// will be cancelled deep inside REmbedInternal, where it's safest
+		if (!ok) args->params["cancelled"] = QVariant (true);
 	} else if ((type == RCallbackArgs::RShowFiles) || (type == RCallbackArgs::REditFiles)) {
-		if ((type == RCallbackArgs::RShowFiles) && (QString (*(args->chars_d)) == "rkwardhtml")) {
-			// not to worry, just some help file to display
-			// TODO: maybe move this to ShowEditTextFileAgent instead
-			for (int n=0; n < args->int_a; ++n) {
-				RKWardMainWindow::getMain ()->openHTML (KUrl (args->chars_a[n]));
-			}
-		} else {
-			ShowEditTextFileAgent::showEditFiles (args);
-			return;
-		}
-	} else if (type ==RCallbackArgs::RChooseFile) {
+		ShowEditTextFileAgent::showEditFiles (args);
+		return;		// we are not done, yet!
+	} else if (type == RCallbackArgs::RChooseFile) {
 		QString filename;
-		if (args->int_a) {
+		if (args->params["new"].toBool ()) {
 			filename = KFileDialog::getSaveFileName ();
 		} else {
 			filename = KFileDialog::getOpenFileName ();
 		}
-		filename = filename.left (args->int_b - 2);
-		args->int_c = filename.length ();
-		qstrcpy (*(args->chars_a), filename.toLatin1 ());
-	} else if (type ==RCallbackArgs::RSuicide) {
-		QString message = i18n ("The R engine has encountered a fatal error:\n") + QString (*(args->chars_a));
-		message += i18n ("It will be shut down immediately. This means, you can not use any more functions that rely on the R backend. I.e. you can do hardly anything at all, not even save the workspace. What you can do, however, is save any open command-files, the output, or copy data out of open data editors. Quit RKWard after that. Sorry!");
-		KMessageBox::error (0, message, i18n ("R engine has died"));
-		r_thread->kill ();
-	} else if (type ==RCallbackArgs::RCleanUp) {
-		QString message = i18n ("The R engine has shut down with status: ") + QString::number (args->int_a);
-		message += i18n ("\nIt will be shut down immediately. This means, you can not use any more functions that rely on the R backend. I.e. you can do hardly anything at all, not even save the workspace. Hopefully, however, R has already saved the workspace. What you can do, however, is save any open command-files, the output, or copy data out of open data editors. Quit RKWard after that.\nSince this should never happen, please write a mail to rkward-devel@lists.sourceforge.net, and tell us, what you were trying to do, when this happened. Sorry!");
+		args->params["result"] = QVariant (filename);
+	} else if (type == RCallbackArgs::RBackendExit) {
+		QString message = args->params["message"].toString ();
+		message += i18n ("\nIt will be shut down immediately. This means, you can not use any more functions that rely on the R backend. I.e. you can do hardly anything at all, not even save the workspace (but if you're lucky, R already did that). What you can do, however, is save any open command-files, the output, or copy data out of open data editors. Quit RKWard after that.\nSince this should never happen, please write a mail to rkward-devel@lists.sourceforge.net, and tell us, what you were trying to do, when this happened. Sorry!");
 		KMessageBox::error (0, message, i18n ("R engine has died"));
 		r_thread->kill ();
 	}
