@@ -90,11 +90,17 @@
 	}
 }
 
+# should this really be public?
 "rk.sync" <- function (x) {
 	object <- deparse (substitute (x))
 	.rk.do.call ("sync", object)
 }
 
+# should this really be public?
+"rk.sync.global" <- function () {
+	.rk.do.call("syncglobal", ls (envir=globalenv (), all.names=TRUE))
+}
+
 "rk.edit" <- function (x) {
 	object <- deparse (substitute (x))
 	.rk.do.call ("edit", object)
@@ -110,6 +116,7 @@
 
 "rk.set.output.html.file" <- function (x) {
 	stopifnot (is.character (x))
+	.rk.do.call ("set.output.file", x);
 	assign (".rk.output.html.file", x, as.environment ("package:rkward"))
 }
 
@@ -304,3 +311,57 @@
 
 	x
 }
+
+"rk.edit.files" <- function (file = file, title = file, name = NULL)
+{
+	if (!is.character (file)) {
+		nfile = tempfile()
+		env = environment (file)
+		dput (file, file=nfile)
+		.Call("rk.edit.files", nfile, title, name)
+		x <- dget (nfile)
+		environment (x) <- env
+		return (x)
+	}
+	invisible (.Call ("rk.edit.files", file, title, name))
+}
+
+"rk.show.files" <- function (file = file, title = file, wtitle = NULL, delete=FALSE)
+{
+	invisible (.Call ("rk.show.files", as.character (file), as.character (title), as.character (wtitle), delete))
+}
+
+"rk.show.html" <- function (url) {
+	invisible (.rk.do.call ("showHTML", as.character (url)));
+}
+
+"rk.call.plugin" <- function (plugin, ..., submit.mode = c ("manual", "auto", "submit")) {
+	# prepare arguments
+	settings <- list (...)
+	callstrings <- list ()
+	callstrings[1] <- plugin
+	callstrings[2] <- match.arg (submit.mode)
+	if (length (settings) > 0) {
+		for (i in 1:length(settings)) {
+			# properly passing on escaped characters is a pain. This seems to work.
+			deparsed <- deparse (settings[[i]])
+			deparsed_unquoted <- substr (deparsed, 2, nchar (deparsed) - 1)
+			callstrings[i + 2] <- paste(names(settings)[i], deparsed_unquoted, 
+			sep = "=")
+		}
+	}
+
+	# do call
+	res <- .rk.do.call ("doPlugin", callstrings)
+
+	# handle result
+	if (!is.null (res)) {
+		if (res$type == "warning") {
+			warning (res$message)
+		} else {
+			stop (res$message)
+		}
+	}
+
+	invisible (TRUE)
+}
