@@ -2,7 +2,7 @@
                           rthread  -  description
                              -------------------
     begin                : Mon Aug 2 2004
-    copyright            : (C) 2004, 2006, 2007, 2008 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,6 +22,7 @@
 #include "../settings/rksettingsmodulegeneral.h"
 #include "../rkglobals.h"
 #include "../rkward.h"		// for startup options
+#include "../version.h"
 
 #include "../debug.h"
 
@@ -29,11 +30,13 @@
 #include <klocale.h>
 
 #include <qstring.h>
-#include <qapplication.h>
-#include <QDBusConnection>
 #include <QList>
+#include <QFileInfo>
 
-#include <signal.h>		// needed for pthread_kill
+#ifndef Q_WS_WIN
+#	include <signal.h>		// needed for pthread_kill
+#	include <pthread.h>		// seems to be needed at least on FreeBSD
+#endif
 
 #define MAX_BUF_LENGTH 4000
 
@@ -48,10 +51,10 @@ RThread::RThread () : QThread (), REmbedInternal () {
 	out_buf_len = 0;
 	output_paused = false;
 
-	toplevel_env_names = 0;
-	toplevel_env_count = 0;
-	global_env_toplevel_names = 0;
-	global_env_toplevel_count = 0;
+#ifdef Q_WS_WIN
+	// we hope that on other platforms the default is reasonable
+	setStackSize (0xa00000);	// 10MB as recommended by r_exts-manual
+#endif
 }
 
 RThread::~RThread() {
@@ -59,15 +62,12 @@ RThread::~RThread() {
 }
 
 void RThread::interruptProcessing (bool interrupt) {
-// TODO: find a good #ifdef to use the uncommented version below, if on a system without pthreads
-	if (interrupt) {
-		pthread_kill ((pthread_t) thread_id, SIGINT);
-	}
-/*	if (interrupt) {
-		R_interrupts_pending = 1;
-	} else {
-		R_interrupts_pending = 0;
-	} */
+	if (!interrupt) return;
+#ifdef Q_WS_WIN
+	RK_scheduleIntr ();
+#else
+	pthread_kill ((pthread_t) thread_id, SIGINT);
+#endif
 }
 
 void RThread::run () {
@@ -81,12 +81,14 @@ void RThread::run () {
 	// in RInterface::RInterface() we create a fake RCommand to capture all the output/errors during startup
 	MUTEX_LOCK;
 	current_command = RCommandStack::regular_stack->currentCommand ();
+	all_current_commands.append (current_command);
 	RK_ASSERT (current_command);
 	MUTEX_UNLOCK;
 
 	if ((err = initialize ())) {
 		int* err_c = new int;
 		*err_c = err;
+		flushOutput ();		// to make errors/warnings available to the main thread
 		qApp->postEvent (RKGlobals::rInterface (), new RKRBackendEvent (RKRBackendEvent::RStartupError, err_c));
 	}
 	qApp->postEvent (RKGlobals::rInterface (), new RKRBackendEvent (RKRBackendEvent::RStarted));
@@ -99,6 +101,7 @@ void RThread::run () {
 	MUTEX_LOCK;
 	checkObjectUpdatesNeeded (true);
 	RCommandStack::regular_stack->pop ();	// remove the fake command
+	all_current_commands.pop_back ();
 	notifyCommandDone (current_command);
 	MUTEX_UNLOCK;
 
@@ -167,10 +170,6 @@ void RThread::doCommand (RCommand *command) {
 		
 		RK_DO (qDebug ("running command: %s", ccommand.toLatin1().data ()), RBACKEND, DL_DEBUG);
 	
-		if (command->type () & RCommand::DirectToOutput) {
-			runCommandInternal (".rk.cat.output (\"<hr>\\n\")", &error, false);
-			RK_ASSERT (!error);
-		}
 		command->status |= RCommand::Running;	// it is important that this happens before the Mutex is unlocked!
 		RCommandStackModel::getModel ()->itemChange (command);
 
@@ -234,6 +233,11 @@ void RThread::doCommand (RCommand *command) {
 				RK_ASSERT (!error);
 			}
 		}
+		if (!(command->type () & RCommand::Sync)) {
+			MUTEX_UNLOCK;
+			checkNotifyOutputTouched ();
+			MUTEX_LOCK;
+		}
 	
 		if (error) {
 			RK_DO (qDebug ("- error message was: '%s'", command->error ().toLatin1 ().data ()), RBACKEND, dl);
@@ -369,10 +373,14 @@ void RThread::handleError (QString *call, int call_length) {
 	waitIfOutputPaused ();
 
 	MUTEX_LOCK;
-	// Unfortunately, errors still get printed to the output. We try this crude method for the time being:
+	// Unfortunately, errors still get printed to the output, UNLESS a sink() is in place. We try this crude method for the time being:
 	flushOutput ();
 	if (current_command) {
-		current_command->output_list.last ()->type = ROutput::Error;
+		if (!current_command->output_list.isEmpty ()) {
+			if (current_command->output_list.last ()->output == call[0]) {
+				current_command->output_list.last ()->type = ROutput::Error;
+			}
+		}
 		current_command->status |= RCommand::HasError;
 	}
 
@@ -380,30 +388,46 @@ void RThread::handleError (QString *call, int call_length) {
 	MUTEX_UNLOCK;
 }
 
-void RThread::handleSubstackCall (QString *call, int call_length) {
+void RThread::checkNotifyOutputTouched () {
 	RK_TRACE (RBACKEND);
 
-	if (call_length == 2) {		// schedule symbol update for later
+	QFileInfo info (active_output_file);
+	if (info.exists ()) {
+		if ((!output_last_modified.isValid ()) || (output_last_modified < info.lastModified ())) {
+			output_last_modified = info.lastModified ();
+			handleSubstackCall (QStringList () << "refreshOutput" << active_output_file);
+		}
+	}
+}
+
+void RThread::handleSubstackCall (QStringList &call) {
+	RK_TRACE (RBACKEND);
+
+	if (call.count () == 2) {		// schedule symbol update for later
 		if (call[0] == "ws") {
 			RK_ASSERT (current_command);
 			if ((current_command->type () & RCommand::ObjectListUpdate) || (!(current_command->type () & RCommand::Sync))) {		// ignore Sync commands that are not flagged as ObjectListUpdate
 				if (!changed_symbol_names.contains (call[1])) changed_symbol_names.append (call[1]);
 			}
 			return;
+		} else if (call[0] == "set.output.file") {
+			checkNotifyOutputTouched ();
+			active_output_file = call[1];
+			output_last_modified = QFileInfo (active_output_file).lastModified ();
+			return;
 		}
 	}
 
 	RCommand *prev_command = current_command;
-	REvalRequest *request = new REvalRequest;
-	request->call = call;
-	request->call_length = call_length;
+	REvalRequest request;
+	request.call = call;
 	MUTEX_LOCK;
 	flushOutput ();
 	RCommandStack *reply_stack = new RCommandStack (prev_command);
-	request->in_chain = reply_stack->startChain (reply_stack);
+	request.in_chain = reply_stack->startChain (reply_stack);
 	MUTEX_UNLOCK;
 
-	RKRBackendEvent* event = new RKRBackendEvent (RKRBackendEvent::REvalRequest, request);
+	RKRBackendEvent* event = new RKRBackendEvent (RKRBackendEvent::REvalRequest, &request);
 	qApp->postEvent (RKGlobals::rInterface (), event);
 	
 	bool done = false;
@@ -443,7 +467,6 @@ void RThread::handleSubstackCall (QString *call, int call_length) {
 	}
 
 	MUTEX_LOCK;
-	delete request;
 	delete reply_stack;
 	MUTEX_UNLOCK;
 }
@@ -497,16 +520,11 @@ int RThread::initialize () {
 	
 	runCommandInternal ("library (\"rkward\")\n", &error);
 	if (error) status |= LibLoadFail;
-	unsigned int c;
-	QString *paths = getCommandAsStringVector ("library.dynam (\"rkward\", \"rkward\")[[\"path\"]]\n", &c, &error);
-	if ((error) || (c != 1)) {
-		status |= LibLoadFail;
-	} else {
-		if (!registerFunctions (paths[0].toLocal8Bit ())) status |= LibLoadFail;
-	}
-	delete [] paths;
+	runCommandInternal (QString ("stopifnot(.rk.app.version==\"%1\")\n").arg (VERSION), &error);
+	if (error) status |= LibLoadFail;
 
 // find out about standard library locations
+	unsigned int c;
 	QString *standardliblocs = getCommandAsStringVector (".libPaths ()\n", &c, &error);
 	if (error) status |= OtherFail;
 	for (unsigned int i = 0; i < c; ++i) {
@@ -529,9 +547,6 @@ int RThread::initialize () {
 	if (error) status |= SinkFail;
 	runCommandInternal ("rk.set.output.html.file (\"" + RKSettingsModuleGeneral::filesPath () + "/rk_out.html\")\n", &error);
 	if (error) status |= SinkFail;
-	runCommandInternal ("options (htmlhelp=TRUE); options (browser=\"qdbus " + QDBusConnection::sessionBus ().baseService () + " /MainApplication net.sf.rkward.openHTMLHelp\")", &error);
-	if (error) status |= OtherFail;
-	// TODO: error-handling?
 
 	MUTEX_LOCK;
 	flushOutput ();
@@ -558,53 +573,57 @@ void RThread::checkObjectUpdatesNeeded (bool check_list) {
 	// TODO: avoid parsing this over and over again
 		RK_DO (qDebug ("checkObjectUpdatesNeeded: getting search list"), RBACKEND, DL_TRACE);
 		strings = getCommandAsStringVector ("search ()\n", &count, &error);
-		if (count != toplevel_env_count) {
+		if ((int) count != toplevel_env_names.count ()) {
 			search_update_needed = true;
 		} else {
-			for (unsigned int i = 0; i < toplevel_env_count; ++i) {
+			for (unsigned int i = 0; i < count; ++i) {
+				// order is important in the search path
 				if (toplevel_env_names[i] != strings[i]) {
 					search_update_needed = true;
 					break;
 				}
 			}
 		}
-		delete [] toplevel_env_names;
-		toplevel_env_names = strings;
-		toplevel_env_count = count;
+		if (search_update_needed) {
+			toplevel_env_names.clear ();
+			for (unsigned int i = 0; i < count; ++i) {
+				toplevel_env_names.append (strings[i]);
+			}
+		}
 	
 	// TODO: avoid parsing this over and over again
 		RK_DO (qDebug ("checkObjectUpdatesNeeded: getting globalenv symbols"), RBACKEND, DL_TRACE);
 		strings = getCommandAsStringVector ("ls (globalenv (), all.names=TRUE)\n", &count, &error);
-		if (count != global_env_toplevel_count) {
+		if ((int) count != global_env_toplevel_names.count ()) {
 			globalenv_update_needed = true;
 		} else {
-			for (unsigned int i = 0; i < global_env_toplevel_count; ++i) {
-				bool found = false;
-				for (unsigned int j = 0; j < global_env_toplevel_count; ++j) {
-					if (global_env_toplevel_names[j] == strings[i]) {
-						found = true;
-						break;
-					}
-				}
-				if (!found) {
+			for (unsigned int i = 0; i < count; ++i) {
+				// order is not important in the symbol list
+				if (!global_env_toplevel_names.contains (strings[i])) {
 					globalenv_update_needed = true;
 					break;
 				}
 			}
 		}
-		delete [] global_env_toplevel_names;
-		global_env_toplevel_names = strings;
-		global_env_toplevel_count = count;
+		if (globalenv_update_needed) {
+			global_env_toplevel_names.clear ();
+			for (unsigned int i = 0; i < count; ++i) {
+				global_env_toplevel_names.append (strings[i]);
+			}
+		}
 	
 		if (search_update_needed) {	// this includes an update of the globalenv, even if not needed
-			QString call = "syncall";
 			MUTEX_UNLOCK;
-			handleSubstackCall (&call, 1);
+			QStringList call = toplevel_env_names;
+			call.prepend ("syncenvs");	// should be faster than the reverse
+			handleSubstackCall (call);
 			MUTEX_LOCK;
-		} else if (globalenv_update_needed) {
-			QString call = "syncglobal";
+		} 
+		if (globalenv_update_needed) {
 			MUTEX_UNLOCK;
-			handleSubstackCall (&call, 1);
+			QStringList call = global_env_toplevel_names;
+			call.prepend ("syncglobal");	// should be faster than the reverse
+			handleSubstackCall (call);
 			MUTEX_LOCK;
 		}
 	}
@@ -615,18 +634,11 @@ void RThread::checkObjectUpdatesNeeded (bool check_list) {
 	}
 
 	if (!changed_symbol_names.isEmpty ()) {
-		int call_length = changed_symbol_names.count () + 1;
-		QString *call = new QString[call_length];
-		call[0] = "sync";
-		int i = 1;
-		for (QStringList::const_iterator it = changed_symbol_names.constBegin (); it != changed_symbol_names.constEnd (); ++it) {
-			call[i++] = *it;
-		}
-		RK_ASSERT (i == call_length);
+		QStringList call = changed_symbol_names;
+		call.prepend (QString ("sync"));	// should be faster than reverse
 		MUTEX_UNLOCK;
-		handleSubstackCall (call, call_length);
+		handleSubstackCall (call);
 		MUTEX_LOCK;
-		delete [] call;
 		changed_symbol_names.clear ();
 	}
 }
