@@ -2,7 +2,7 @@
                           rkward.cpp  -  description
                              -------------------
     begin                : Tue Oct 29 20:06:08 CET 2002
-    copyright            : (C) 2002, 2005, 2006, 2007, 2008 by Thomas Friedrichsmeier 
+    copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier 
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -46,6 +46,7 @@
 #include <kactioncollection.h>
 #include <krecentfilesaction.h>
 #include <khbox.h>
+#include <ktoolbar.h>
 
 // application specific includes
 #include "rkward.h"
@@ -59,6 +60,7 @@
 #include "core/robjectlist.h"
 #include "core/renvironmentobject.h"
 #include "misc/rkstandardicons.h"
+#include "misc/rkcommonfunctions.h"
 #include "rkglobals.h"
 #include "robjectbrowser.h"
 #include "dialogs/startupdialog.h"
@@ -77,6 +79,7 @@
 #include "windows/rkfilebrowser.h"
 #include "rkconsole.h"
 #include "debug.h"
+#include "version.h"
 
 
 #include "agents/showedittextfileagent.h"	// TODO: see below: needed purely for linking!
@@ -95,18 +98,6 @@ void bogusCalls () {
 	new RKEditObjectAgent (QStringList (), 0);
 }
 
-/** DBUS interface **/
-
-RKWardDBUSInterface::RKWardDBUSInterface (QApplication *application) : QDBusAbstractAdaptor(application) {
-	RK_TRACE (APP);
-}
-
-void RKWardDBUSInterface::openHTMLHelp (const QString &url) {
-	RK_TRACE (APP);
-
-	RKWorkplace::mainWorkplace ()->openHelpWindow (url);
-}
-
 /** Main window **/
 
 //static
@@ -156,12 +147,6 @@ RKWardMainWindow::RKWardMainWindow (RKWardStartupOptions *options) : KParts::Mai
 	// When the manager says the active part changes,
 	// the builder updates (recreates) the GUI
 	connect (partManager (), SIGNAL (activePartChanged (KParts::Part *)), this, SLOT (partChanged (KParts::Part *)));
-
-	// create the DBUS adaptor:
-	new RKWardDBUSInterface (qApp);
-
-	// connect to D-Bus and register as an object:
-	QDBusConnection::sessionBus ().registerObject ("/MainApplication", qApp);
 }
 
 RKWardMainWindow::~RKWardMainWindow() {
@@ -196,6 +181,16 @@ void RKWardMainWindow::closeEvent (QCloseEvent *e) {
 void RKWardMainWindow::doPostInit () {
 	RK_TRACE (APP);
 
+	// Check installation first
+	QFile resource_ver (RKCommonFunctions::getRKWardDataDir () + "resource.ver");
+	if (!(resource_ver.open (QIODevice::ReadOnly) && (resource_ver.read (100).trimmed () == VERSION))) {
+		KMessageBox::error (this, i18n ("<p>RKWard either could not find its resource files at all, or only an old version of those files. The most likely cause is that the last installation failed to place the files in the correct place. This can lead to all sorts of problems, from single missing features to complete failure to function.</p><p><b>You should quit RKWard, now, and fix your installation</b>. For help with that, see <a href=\"http://p.sf.net/rkward/compiling\">http://p.sf.net/rkward/compiling</a>.</p>"), i18n ("Broken installation"), KMessageBox::Notify | KMessageBox::AllowLink);
+	}
+
+#ifdef Q_WS_WIN
+	KMessageBox::information (this, i18n ("<p>Please note that RKWard on is not well tested on Windows, yet.</p><p>There are all sorts of known issues, and issues yet to be reported. If you'd like to help: <a href=\"http://p.sf.net/rkward/contact\">http://p.sf.net/rkward/contact</a>.</p>"), i18n ("RKWard on Windows"), "rkward_on_windows", KMessageBox::Notify | KMessageBox::AllowLink);
+#endif
+
 	setUpdatesEnabled (false);
 
 	readOptions();
@@ -208,10 +203,8 @@ void RKWardMainWindow::doPostInit () {
 
 	// startup options will be deleted from the R thread (TODO correct this!), so we need to copy the initial_url here, or run into race conditions
 	KUrl open_url = startup_options->initial_url;
+	QString evaluate_code = startup_options->evaluate;
 	startR ();
-
-	QString dummy = i18n ("RKWard has made great progress in the past few months and it is already helpful for many tasks, but some features may be lacking. You can help us by filing bug reports, feature requests, or providing feedback in any other form. Please visit http://rkward.sourceforge.net for more information.");
-	KMessageBox::information (this, dummy, i18n("What to expect of RKWard"), "state_of_rkward");
 	
 	initPlugins ();
 
@@ -237,8 +230,6 @@ void RKWardMainWindow::doPostInit () {
 	setUpdatesEnabled (true);
 	show ();
 
-	RKOutputWindow::initialize ();
-
 	if (!open_url.isEmpty()) {
 		openWorkspace (open_url);
 	} else {
@@ -262,6 +253,8 @@ void RKWardMainWindow::doPostInit () {
 		toplevel_actions->showRKWardHelp ();
 	}
 
+	if (!evaluate_code.isEmpty ()) RKConsole::pipeUserCommand (evaluate_code);
+
 	setCaption (QString ());	// our version of setCaption takes care of creating a correct caption, so we do not need to provide it here
 }
 
@@ -391,9 +384,41 @@ void RKWardMainWindow::initActions()
 	run_menu_dummy->setEnabled (false);
 }
 
+void RKWardMainWindow::changeEvent (QEvent *e) {
+	RK_TRACE (APP);
+
+	// see RKWardMainWindow::partChanged() for a detailed comment
+	if ((e->type () == QEvent::ActivationChange) && isActiveWindow () && isVisible ()) {
+		RKMDIWindow *active = RKWorkplace::mainWorkplace ()->activeWindow (RKMDIWindow::Attached);
+		toplevel_actions->reloadXML ();
+		createGUI (0);
+		if (active) {
+			active->fixupPartGUI (true);
+			createGUI (active->getPart ());
+		}
+		// NOTE: KXMLGUIFactory::refreshActionProperties(), which would be a simple repalcement for the above, seems to cause crashes, at least with KDElibs 4.2.2: http://sourceforge.net/tracker/?func=detail&atid=459007&aid=2828002&group_id=50231
+	}
+
+	KParts::MainWindow::changeEvent (e);
+}
+
 void RKWardMainWindow::partChanged (KParts::Part *part) {
 	RK_TRACE (APP);
 
+	/* Changes in shortcut-settings are not automatically synced between all action collections. Therefore, if we change between parts with the same actions, the new part simply does not have any changes that happend while the other part was active.
+	Reliably detecting such changes seems impossible (QAction::changed() gets emitted all the time for all sorts of non-interesting changes as well). So what we do is simply make sure we sync the KXMLGUI-files each time we switch to a new part. Fortunately, the performance impact appears to be small. */
+
+	// first: find out, which window is about to be activated
+	if (part) {
+		QList<RKMDIWindow*> list = RKWorkplace::mainWorkplace ()->getObjectList ();
+		for (int i = 0; i < list.count (); ++i) {
+			if (list[i]->getPart () == part) {
+				// now reload XML and apply customizations
+				list[i]->fixupPartGUI (true);
+				break;
+			}
+		}
+	}
 	createGUI (part);
 
 	if (!guiFactory ()) {
@@ -454,6 +479,13 @@ void RKWardMainWindow::saveOptions () {
 
 void RKWardMainWindow::readOptions () {
 	RK_TRACE (APP);
+	// first make sure to give the global defaults a chance, if needed.
+	// TODO: Why don't the toolbars honor the global style automatically?
+	QList<KToolBar*> tool_bars = toolBars ();
+	for (int i=0; i < tool_bars.size (); ++i) {
+		tool_bars[i]->setToolButtonStyle (KToolBar::toolButtonStyleSetting ());
+	}
+
 	KConfig *config = KGlobal::config ().data ();
 
 	applyMainWindowSettings (config->group ("main window options"), true);
@@ -524,7 +556,13 @@ void RKWardMainWindow::fileOpenNoSave (const KUrl &url) {
 	slotSetStatusBarText(i18n("Opening workspace..."));
 	KUrl lurl = url;
 	if (lurl.isEmpty ()) {
+#ifdef Q_WS_WIN
+	// getOpenUrl(KUrl("kfiledialog:///<rfiles>"), ...) causes a hang on windows (KDElibs 4.2.3).
+#	warning Track this bug down and/or report it
+		lurl = KFileDialog::getOpenUrl (KUrl (), i18n("*|All files"), this, i18n("Open File..."));
+#else
 		lurl = KFileDialog::getOpenUrl (KUrl ("kfiledialog:///<rfiles>"), i18n("*|All files"), this, i18n("Open File..."));
+#endif
 	}
 	if (!lurl.isEmpty ()) {
 		openWorkspace (lurl);
@@ -663,9 +701,14 @@ void RKWardMainWindow::slotOpenCommandEditor () {
 	RK_TRACE (APP);
 	KUrl::List urls;
 	KUrl::List::const_iterator it;
-	
-	urls = KFileDialog::getOpenUrls (KUrl ("kfiledialog:///<rfiles>"), "*.R *.r *.S *.s *.q|R Script Files (*.R *.r *.S *.s *.q)\n*.*|All Files (*.*)", this, i18n ("Open command file(s)"));
 
+#ifdef Q_WS_WIN
+	// getOpenUrls(KUrl("kfiledialog:///<rfiles>"), ...) causes a hang on windows (KDElibs 4.2.3).
+#	warning Track this bug down and/or report it
+	urls = KFileDialog::getOpenUrls (KUrl (), "*.R *.r *.S *.s *.q|R Script Files (*.R *.r *.S *.s *.q)\n*.*|All Files (*.*)", this, i18n ("Open command file(s)"));
+#else
+	urls = KFileDialog::getOpenUrls (KUrl ("kfiledialog:///<rfiles>"), "*.R *.r *.S *.s *.q|R Script Files (*.R *.r *.S *.s *.q)\n*.*|All Files (*.*)", this, i18n ("Open command file(s)"));
+#endif
 	for (it = urls.begin() ; it != urls.end() ; ++it) {
 		slotOpenCommandEditor (*it);
 	}
