@@ -2,7 +2,7 @@
 			rkward.h  -  description
 			-------------------
 begin                : Tue Oct 29 20:06:08 CET 2002 
-copyright            : (C) 2002, 2005, 2006, 2007, 2008 by Thomas Friedrichsmeier 
+copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier 
 email                : tfry@users.sourceforge.net
 ***************************************************************************/
 
@@ -18,31 +18,6 @@ email                : tfry@users.sourceforge.net
 #ifndef RKWARD_H
 #define RKWARD_H
 
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include<QDBusAbstractAdaptor>
-
-class QApplication;
-
-/** This base provides the DBUS-Interface for RKWardMainWindow */
-class RKWardDBUSInterface : virtual public QDBusAbstractAdaptor {
-	Q_OBJECT
-	Q_CLASSINFO("D-Bus Interface", "net.sf.rkward")
-public:
-	RKWardDBUSInterface (QApplication *application);
-	~RKWardDBUSInterface () {};
-public slots:
-	/** Open the given html help page */
-	void openHTMLHelp (const QString &url);
-};
-
-
-// include files for Qt
-
-// include files for KDE 
 #include <kapplication.h>
 #include <kaction.h>
 #include <kurl.h>
@@ -69,6 +44,7 @@ class KSqueezedTextLabel;
 struct RKWardStartupOptions {
 	KUrl initial_url;	/**< The workspace file to load on startup. If empty, show a dialog asking what to do. **/
 	bool no_stack_check;	/**< Disable R C stack checking */
+	QString evaluate;	/**< R code to run after startup */
 };
 
 /**
@@ -119,6 +95,8 @@ protected:
 	void initStatusBar();
 	/** reimplemented from KMainWindow to call our doQueryClose (), and then (if quitting was not cancelled), invoke an RKQuitAgent to wait for the R-backend to finish up before actually quitting. */
 	virtual void closeEvent (QCloseEvent *e);
+/** may need to reload the active window's XMLGUI definition */
+	void changeEvent (QEvent *);
 signals:
 	void aboutToQuitRKWard ();
 public slots:
@@ -178,33 +156,33 @@ private:
 	KParts::PartManager *part_manager;
 
 	// KAction pointers to enable/disable actions
-	QAction* fileOpen;
+	KAction* fileOpen;
 	KRecentFilesAction* fileOpenRecent;
 	
-	QAction* fileOpenWorkspace;
+	KAction* fileOpenWorkspace;
 	KRecentFilesAction* fileOpenRecentWorkspace;
-	QAction* fileSaveWorkspace;
-	QAction* fileSaveWorkspaceAs;
-	QAction* fileQuit;
-	QAction* file_load_libs;
-	QAction* close_all_editors;
-	QAction* new_data_frame;
-	QAction* new_command_editor;
+	KAction* fileSaveWorkspace;
+	KAction* fileSaveWorkspaceAs;
+	KAction* fileQuit;
+	KAction* file_load_libs;
+	KAction* close_all_editors;
+	KAction* new_data_frame;
+	KAction* new_command_editor;
 	
-	QAction* editUndo;
-	QAction* editRedo;
+	KAction* editUndo;
+	KAction* editRedo;
 
-	QAction* window_close_all;
-	QAction* window_detach;
+	KAction* window_close_all;
+	KAction* window_detach;
 	
-	QAction* configure;
+	KAction* configure;
 
 	/** used so that if the menu is empty, there is a note in it, explaining that fact */
-	QAction* edit_menu_dummy;
+	KAction* edit_menu_dummy;
 	/** used so that if the menu is empty, there is a note in it, explaining that fact */
-	QAction* view_menu_dummy;
+	KAction* view_menu_dummy;
 	/** used so that if the menu is empty, there is a note in it, explaining that fact */
-	QAction* run_menu_dummy;
+	KAction* run_menu_dummy;
 
 	friend class RKSettingsModule;
 	friend class RKSettingsModulePlugins;
