@@ -73,7 +73,7 @@ bool PHPBackend::initialize (RKComponentPropertyCode *code_property, bool add_he
 	connect (php_process, SIGNAL (error(QProcess::ProcessError)), this, SLOT (processError(QProcess::ProcessError)));
 	connect (php_process, SIGNAL (finished(int,QProcess::ExitStatus)), this, SLOT (processDead(int,QProcess::ExitStatus)));
 
-	php_process->start (RKSettingsModulePHP::phpBin() + " -c " + php_ini + " " + common_php);
+	php_process->start ("\"" + RKSettingsModulePHP::phpBin() + "\"" + " -c \"" + php_ini + "\" \"" + common_php + "\"");
 
 	doing_command = startup_done = false;
 	busy = true;
@@ -186,7 +186,11 @@ void PHPBackend::gotOutput () {
 		if (request == "requesting code") {
 			startup_done = true;
 			RK_DO (qDebug ("got type: %d, stack %d", current_type, command_stack.count ()), PHP, DL_DEBUG);
+#ifdef Q_OS_WIN
+			commandFinished (_output.replace ("\r\n", "\n"));
+#else
 			commandFinished (_output);
+#endif
 			_output = QString::null;
 		} else if (request.startsWith ("requesting data:")) {
 			QString requested_object = request.remove ("requesting data:");
