@@ -31,7 +31,7 @@ ScriptBackend::~ScriptBackend () {
 
 void ScriptBackend::callFunction (const QString &function, int flags, int type) {
 	RK_TRACE (PHP);
-	RK_DO (qDebug ("callFunction %s", function.latin1 ()), PHP, DL_DEBUG);
+	RK_DO (qDebug ("callFunction %s", function.toLatin1 ().data ()), PHP, DL_DEBUG);
 
 	ScriptCommand *command = new ScriptCommand;
 	command->command = function;
@@ -63,7 +63,7 @@ void ScriptBackend::invalidateCalls (int type) {
 		current_type = Ignore;
 	}
 
-	QValueList<ScriptCommand *>::iterator it = command_stack.begin ();
+	QLinkedList<ScriptCommand *>::iterator it = command_stack.begin ();
 	while (it != command_stack.end ()) {
 		if ((*it)->type == type) {
 			delete (*it);
