@@ -2,7 +2,7 @@
                           rksettingsmodulegeneral  -  description
                              -------------------
     begin                : Fri Jul 30 2004
-    copyright            : (C) 2004, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2007, 2008 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -27,7 +27,9 @@
 #include <qcombobox.h>
 #include <qcheckbox.h>
 #include <qbuttongroup.h>
+#include <qgroupbox.h>
 #include <qradiobutton.h>
+#include <QVBoxLayout>
 
 #include "../misc/getfilenamewidget.h"
 #include "../misc/rkspinbox.h"
@@ -41,26 +43,32 @@ StartupDialog::Result RKSettingsModuleGeneral::startup_action;
 RKSettingsModuleGeneral::WorkplaceSaveMode RKSettingsModuleGeneral::workplace_save_mode;
 bool RKSettingsModuleGeneral::show_help_on_startup;
 int RKSettingsModuleGeneral::warn_size_object_edit;
+RKSettingsModuleGeneral::RKMDIFocusPolicy RKSettingsModuleGeneral::mdi_focus_policy;
 
 RKSettingsModuleGeneral::RKSettingsModuleGeneral (RKSettings *gui, QWidget *parent) : RKSettingsModule (gui, parent) {
 	RK_TRACE (SETTINGS);
 
-	QVBoxLayout *main_vbox = new QVBoxLayout (this, RKGlobals::marginHint ());
+	QVBoxLayout *main_vbox = new QVBoxLayout (this);
 	QLabel *label = new QLabel (i18n ("Settings marked with (*) do not take effect until you restart RKWard"), this);
-	label->setAlignment (Qt::AlignAuto | Qt::AlignVCenter | Qt::ExpandTabs | Qt::WordBreak);
+	label->setWordWrap (true);
 	main_vbox->addWidget (label);
 
+	main_vbox->addSpacing (2*RKGlobals::spacingHint ());
+
 	files_choser = new GetFileNameWidget (this, GetFileNameWidget::ExistingDirectory, i18n ("Directory where the logfiles should be kept (*)"), QString::null, new_files_path);
 	connect (files_choser, SIGNAL (locationChanged ()), this, SLOT (pathChanged ()));
 	main_vbox->addWidget (files_choser);
 
+	main_vbox->addSpacing (2*RKGlobals::spacingHint ());
+
 	main_vbox->addWidget (new QLabel (i18n ("Startup Action (*)"), this));
-	startup_action_choser = new QComboBox (false, this);
-	startup_action_choser->insertItem (i18n ("Start with an empty workspace"), StartupDialog::EmptyWorkspace);
-	startup_action_choser->insertItem (i18n ("Start with an empty table"), StartupDialog::EmptyTable);
-	startup_action_choser->insertItem (i18n ("Ask for a file to open"), StartupDialog::ChoseFile);
-	startup_action_choser->insertItem (i18n ("Show selection dialog (default)"), StartupDialog::NoSavedSetting);
-	startup_action_choser->setCurrentItem (startup_action);
+	startup_action_choser = new QComboBox (this);
+	startup_action_choser->setEditable (false);
+	startup_action_choser->insertItem (StartupDialog::EmptyWorkspace, i18n ("Start with an empty workspace"));
+	startup_action_choser->insertItem (StartupDialog::EmptyTable, i18n ("Start with an empty table"));
+	startup_action_choser->insertItem (StartupDialog::ChoseFile, i18n ("Ask for a file to open"));
+	startup_action_choser->insertItem (StartupDialog::NoSavedSetting, i18n ("Show selection dialog (default)"));
+	startup_action_choser->setCurrentIndex (startup_action);
 	connect (startup_action_choser, SIGNAL (activated (int)), this, SLOT (boxChanged (int)));
 	main_vbox->addWidget (startup_action_choser);
 
@@ -72,20 +80,26 @@ RKSettingsModuleGeneral::RKSettingsModuleGeneral (RKSettings *gui, QWidget *pare
 	main_vbox->addSpacing (2*RKGlobals::spacingHint ());
 
 	label = new QLabel (i18n ("The workplace layout (i.e. which script-, data-, help-windows are open) may be saved (and loaded) per R workspace, or independent of the R workspace. Which do you prefer?"), this);
-	label->setAlignment (Qt::AlignAuto | Qt::AlignVCenter | Qt::ExpandTabs | Qt::WordBreak);
+	label->setWordWrap (true);
 	main_vbox->addWidget (label);
 
 	workplace_save_chooser = new QButtonGroup (this);
-	workplace_save_chooser->setColumnLayout (0, Qt::Vertical);
-	workplace_save_chooser->layout()->setSpacing (6);
-	workplace_save_chooser->layout()->setMargin (11);
-	QVBoxLayout *group_layout = new QVBoxLayout(workplace_save_chooser->layout());
-	group_layout->addWidget (new QRadioButton (i18n ("Save/restore with R workspace, when saving/loading R workspace"), workplace_save_chooser));
-	group_layout->addWidget (new QRadioButton (i18n ("Save/restore independent of R workspace (save at end of RKWard session, restore at next start)"), workplace_save_chooser));
-	group_layout->addWidget (new QRadioButton (i18n ("Do not save/restore workplace layout"), workplace_save_chooser));
-	workplace_save_chooser->setButton (static_cast<int> (workplace_save_mode));
-	connect (workplace_save_chooser, SIGNAL (clicked (int)), this, SLOT (boxChanged (int)));
-	main_vbox->addWidget (workplace_save_chooser);
+	QGroupBox* group_box = new QGroupBox (this);
+	QVBoxLayout *group_layout = new QVBoxLayout(group_box);
+
+	QAbstractButton* button;
+	button = new QRadioButton (i18n ("Save/restore with R workspace, when saving/loading R workspace"), group_box);
+	group_layout->addWidget (button);
+	workplace_save_chooser->addButton (button, SaveWorkplaceWithWorkspace);
+	button = new QRadioButton (i18n ("Save/restore independent of R workspace (save at end of RKWard session, restore at next start)"), group_box);
+	group_layout->addWidget (button);
+	workplace_save_chooser->addButton (button, SaveWorkplaceWithSession);
+	button = new QRadioButton (i18n ("Do not save/restore workplace layout"), group_box);
+	group_layout->addWidget (button);
+	workplace_save_chooser->addButton (button, DontSaveWorkplace);	
+	if ((button = workplace_save_chooser->button (workplace_save_mode))) button->setChecked (true);
+	connect (workplace_save_chooser, SIGNAL (buttonClicked (int)), this, SLOT (boxChanged (int)));
+	main_vbox->addWidget (group_box);
 
 	main_vbox->addSpacing (2*RKGlobals::spacingHint ());
 
@@ -97,6 +111,18 @@ RKSettingsModuleGeneral::RKSettingsModuleGeneral (RKSettings *gui, QWidget *pare
 	main_vbox->addWidget (label);
 	main_vbox->addWidget (warn_size_object_edit_box);
 
+	main_vbox->addSpacing (2*RKGlobals::spacingHint ());
+
+	label = new QLabel (i18n ("MDI window focus behavior"), this);
+	mdi_focus_policy_chooser = new QComboBox (this);
+	mdi_focus_policy_chooser->setEditable (false);
+	mdi_focus_policy_chooser->insertItem (RKMDIClickFocus, i18n ("Click to focus"));
+	mdi_focus_policy_chooser->insertItem (RKMDIFocusFollowsMouse, i18n ("Focus follows mouse"));
+	mdi_focus_policy_chooser->setCurrentIndex (mdi_focus_policy);
+	connect (mdi_focus_policy_chooser, SIGNAL (activated (int)), this, SLOT (boxChanged (int)));
+	main_vbox->addWidget (label);
+	main_vbox->addWidget (mdi_focus_policy_chooser);
+
 	main_vbox->addStretch ();
 }
 
@@ -127,14 +153,11 @@ bool RKSettingsModuleGeneral::hasChanges () {
 void RKSettingsModuleGeneral::applyChanges () {
 	RK_TRACE (SETTINGS);
 	new_files_path = files_choser->getLocation ();
-	startup_action = static_cast<StartupDialog::Result> (startup_action_choser->currentItem ());
+	startup_action = static_cast<StartupDialog::Result> (startup_action_choser->currentIndex ());
 	show_help_on_startup = show_help_on_startup_box->isChecked ();
-#if QT_VERSION < 0x030200
-	workplace_save_mode = static_cast<WorkplaceSaveMode> (workplace_save_chooser->id (workplace_save_chooser->selected ()));
-#else
-	workplace_save_mode = static_cast<WorkplaceSaveMode> (workplace_save_chooser->selectedId ());
-#endif
-	warn_size_object_edit = warn_size_object_edit_box->value ();
+	workplace_save_mode = static_cast<WorkplaceSaveMode> (workplace_save_chooser->checkedId ());
+	warn_size_object_edit = warn_size_object_edit_box->intValue ();
+	mdi_focus_policy = static_cast<RKMDIFocusPolicy> (mdi_focus_policy_chooser->currentIndex ());
 }
 
 void RKSettingsModuleGeneral::save (KConfig *config) {
@@ -145,49 +168,57 @@ void RKSettingsModuleGeneral::save (KConfig *config) {
 void RKSettingsModuleGeneral::saveSettings (KConfig *config) {
 	RK_TRACE (SETTINGS);
 
-	config->setGroup ("Logfiles");
-	config->writeEntry ("logfile dir", new_files_path);
+	KConfigGroup cg;
+	cg = config->group ("Logfiles");
+	cg.writeEntry ("logfile dir", new_files_path);
 
-	config->setGroup ("General");
-	config->writeEntry ("startup action", (int) startup_action);
-	config->writeEntry ("show help on startup", show_help_on_startup);
+	cg = config->group ("General");
+	cg.writeEntry ("startup action", (int) startup_action);
+	cg.writeEntry ("show help on startup", show_help_on_startup);
 
-	config->setGroup ("Workplace");
-	config->writeEntry ("save mode", (int) workplace_save_mode);
+	cg = config->group ("Workplace");
+	cg.writeEntry ("save mode", (int) workplace_save_mode);
 
-	config->setGroup ("Editor");
-	config->writeEntry ("large object warning limit", warn_size_object_edit);
+	cg = config->group ("Editor");
+	cg.writeEntry ("large object warning limit", warn_size_object_edit);
+
+	cg = config->group ("MDI");
+	cg.writeEntry ("focus policy", (int) mdi_focus_policy);
 }
 
 void RKSettingsModuleGeneral::loadSettings (KConfig *config) {
 	RK_TRACE (SETTINGS);
 
-	config->setGroup ("Logfiles");
-	files_path = new_files_path = config->readEntry ("logfile dir", QDir ().homeDirPath () + "/.rkward/");
+	KConfigGroup cg;
+	cg = config->group ("Logfiles");
+	files_path = new_files_path = cg.readEntry ("logfile dir", QDir ().homePath () + "/.rkward/");
 
-	config->setGroup ("General");
-	startup_action = (StartupDialog::Result) config->readNumEntry ("startup action", StartupDialog::NoSavedSetting);
-	show_help_on_startup = config->readBoolEntry ("show help on startup", true);
+	cg = config->group ("General");
+	startup_action = (StartupDialog::Result) cg.readEntry ("startup action", (int) StartupDialog::NoSavedSetting);
+	show_help_on_startup = cg.readEntry ("show help on startup", true);
 
-	config->setGroup ("Workplace");
-	workplace_save_mode = (WorkplaceSaveMode) config->readNumEntry ("save mode", SaveWorkplaceWithWorkspace);
+	cg = config->group ("Workplace");
+	workplace_save_mode = (WorkplaceSaveMode) cg.readEntry ("save mode", (int) SaveWorkplaceWithWorkspace);
 
-	config->setGroup ("Editor");
-	warn_size_object_edit = config->readNumEntry ("large object warning limit", 250000);
+	cg = config->group ("Editor");
+	warn_size_object_edit = cg.readEntry ("large object warning limit", 250000);
+
+	cg = config->group ("MDI");
+	mdi_focus_policy = (RKMDIFocusPolicy) cg.readEntry ("focus policy", (int) RKMDIClickFocus);
 }
 
 QString RKSettingsModuleGeneral::getSavedWorkplace (KConfig *config) {
 	RK_TRACE (SETTINGS);
 
-	config->setGroup ("Workplace");
-	return (config->readEntry ("last saved layout", QString::null));
+	KConfigGroup cg = config->group ("Workplace");
+	return (cg.readEntry ("last saved layout", QString ()));
 }
 
 void RKSettingsModuleGeneral::setSavedWorkplace (const QString &description, KConfig *config) {
 	RK_TRACE (SETTINGS);
 
-	config->setGroup ("Workplace");
-	config->writeEntry ("last saved layout", description);
+	KConfigGroup cg = config->group ("Workplace");
+	cg.writeEntry ("last saved layout", description);
 }
 
 #include "rksettingsmodulegeneral.moc"
