@@ -2,7 +2,7 @@
                           rksettingsmoduler  -  description
                              -------------------
     begin                : Wed Jul 28 2004
-    copyright            : (C) 2004, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -51,6 +51,8 @@ public:
 	
 /** generate the commands needed to set the R run time options */
 	static QStringList makeRRunTimeOptionCommands ();
+
+	static int getDefaultWidth () { return options_width; };
 public slots:
 	void boxChanged (int);
 	void pathChanged ();
@@ -67,6 +69,8 @@ private:
 	KIntSpinBox *digits_input;
 	QComboBox *checkbounds_input;
 	QLineEdit *printcmd_input;
+	QComboBox *editor_input;
+	QComboBox *pager_input;
 
 	static QString options_outdec;
 	static int options_width;
@@ -79,6 +83,11 @@ private:
 	static int options_digits;
 	static bool options_checkbounds;
 	static QString options_printcmd;
+	static QString options_editor;
+	static QString options_pager;
+
+// constants
+	static QString builtin_editor;
 };
 
 /**
