@@ -2,7 +2,7 @@
                           detachedwindowcontainer  -  description
                              -------------------
     begin                : Wed Oct 21 2005
-    copyright            : (C) 2005, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -20,6 +20,8 @@
 #include <klocale.h>
 #include <kactioncollection.h>
 #include <kstatusbar.h>
+#include <ktoolbar.h>
+#include <kxmlguifactory.h>
 
 #include <qlayout.h>
 #include <qwidget.h>
@@ -36,28 +38,43 @@
 DetachedWindowContainer::DetachedWindowContainer (RKMDIWindow *widget_to_capture) : KParts::MainWindow  () {
 	RK_TRACE (APP);
 
-	setHelpMenuEnabled (false);
-// create own GUI
-	setXMLFile ("detachedwindowcontainer.rc");
 	actionCollection ()->addAction (KStandardAction::Close, "dwindow_close", this, SLOT(close()));
 
 	QAction *reattach = actionCollection ()->addAction ("dwindow_attach", this, SLOT(slotReattach()));
 	reattach->setText (i18n ("Attach to main window"));
 	reattach->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionAttachWindow));
 
-	RKTopLevelWindowGUI *toplevel_actions = new RKTopLevelWindowGUI (this);
-	insertChildClient (toplevel_actions);
+	setHelpMenuEnabled (false);
+// create own GUI
+	setXMLFile ("detachedwindowcontainer.rc");
+	insertChildClient (toplevel_actions = new RKTopLevelWindowGUI (this));
 	statusBar ()->hide ();
 	createShellGUI ();
 
+// copy main window toolbar settings
+	QMap<QString, Qt::ToolButtonStyle> main_window_toolbar_styles;
+	foreach (KToolBar *bar, RKWardMainWindow::getMain ()->toolBars ()) {
+		main_window_toolbar_styles.insert (bar->objectName (), bar->toolButtonStyle ());
+	}
+
 // capture widget
 	setGeometry (widget_to_capture->frameGeometry ());
+	if (!widget_to_capture->isWindow ()) move (widget_to_capture->mapToGlobal (widget_to_capture->pos ()));
 	widget_to_capture->setParent (this);
 	setCentralWidget (widget_to_capture);
 	widget_to_capture->show ();
 	createGUI (widget_to_capture->getPart ());
 	captured = widget_to_capture;
 
+// sanitize toolbars
+	foreach (KToolBar *bar, toolBars ()) {
+		if (main_window_toolbar_styles.contains (bar->objectName ())) {
+			bar->setToolButtonStyle (main_window_toolbar_styles[bar->objectName ()]);
+		} else {
+			RK_ASSERT (false);
+		}
+	}
+
 // should self-destruct, when child widget is destroyed
 	connect (widget_to_capture, SIGNAL (destroyed (QObject *)), this, SLOT (viewDestroyed (QObject *)));
 	connect (widget_to_capture, SIGNAL (captionChanged (RKMDIWindow *)), this, SLOT (updateCaption (RKMDIWindow *)));
@@ -95,15 +112,7 @@ void DetachedWindowContainer::slotSetStatusBarText (const QString &text) {
 void DetachedWindowContainer::slotReattach () {
 	RK_TRACE (APP);
 
-// we will not handle any more signals from the window
-	disconnect (captured, SIGNAL (destroyed (QObject *)), this, SLOT (viewDestroyed (QObject *)));
-	disconnect (captured, SIGNAL (captionChanged (RKMDIWindow *)), this, SLOT (updateCaption (RKMDIWindow *)));
-
-	captured->setParent (0);
 	RKWorkplace::mainWorkplace ()->attachWindow (captured);
-
-	hide ();
-	deleteLater ();
 }
 
 void DetachedWindowContainer::closeEvent (QCloseEvent *e) {
@@ -116,4 +125,19 @@ void DetachedWindowContainer::closeEvent (QCloseEvent *e) {
 	}
 }
 
+void DetachedWindowContainer::changeEvent (QEvent *e) {
+	RK_TRACE (APP);
+
+	// see RKWardMainWindow::partChanged() for a detailed comment
+	if ((e->type () == QEvent::ActivationChange) && isActiveWindow () && isVisible ()) {
+		captured->fixupPartGUI (true);
+		toplevel_actions->reloadXML ();
+		createGUI (0);
+		createGUI (captured->getPart ());
+		// see RKWardMainWindow::changeEvent() for why KXMLGUIFactory::refreshActionProperties () is not used, instead.
+	}
+
+	KParts::MainWindow::changeEvent (e);
+}
+
 #include "detachedwindowcontainer.moc"
