@@ -2,7 +2,7 @@
                           rkcommandeditorwindow  -  description
                              -------------------
     begin                : Mon Aug 30 2004
-    copyright            : (C) 2004, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -21,6 +21,7 @@
 #include <ktexteditor/configinterface.h>
 #include <ktexteditor/sessionconfiginterface.h>
 #include <ktexteditor/editorchooser.h>
+#include <ktexteditor/modificationinterface.h>
 
 #include <qlayout.h>
 #include <qapplication.h>
@@ -82,18 +83,21 @@ RKCommandEditorWindow::RKCommandEditorWindow (QWidget *parent, bool use_r_highli
 
 	m_doc = editor->createDocument (this);
 	RK_ASSERT (m_doc);
+	// yes, we want to be notified, if the file has changed on disk.
+	// why, oh why is this not the default?
+	// this needs to be set *before* the view is created!
+	KTextEditor::ModificationInterface* em_iface = qobject_cast<KTextEditor::ModificationInterface*> (m_doc);
+	if (em_iface) em_iface->setModifiedOnDiskWarning (true);
+	else RK_ASSERT (false);
 	m_view = m_doc->createView (this);
 
 	setFocusProxy (m_view);
 	setFocusPolicy (Qt::StrongFocus);
 
-	// strip down the khtmlpart's GUI. remove some stuff we definitely don't need.
-	RKCommonFunctions::removeContainers (m_view, QString ("bookmarks,tools_spelling,tools_spelling_from_cursor,tools_spelling_selection,switch_to_cmd_line").split (','), true);
-	RKCommonFunctions::moveContainer (m_view, "Menu", "tools", "edit", true);
-
 	RKCommandEditorWindowPart* part = new RKCommandEditorWindowPart (m_view);
 	part->insertChildClient (m_view);
 	setPart (part);
+	fixupPartGUI (false);
 	initializeActions (part->actionCollection ());
 	initializeActivationSignals ();
 
@@ -140,15 +144,25 @@ RKCommandEditorWindow::~RKCommandEditorWindow () {
 	delete m_doc;
 }
 
+void RKCommandEditorWindow::fixupPartGUI (bool reload) {
+	RK_TRACE (COMMANDEDITOR);
+
+	RKMDIWindow::fixupPartGUI (reload);
+
+	// strip down the katepart's GUI. remove some stuff we definitely don't need.
+	RKCommonFunctions::removeContainers (m_view, QString ("bookmarks,tools_spelling,tools_spelling_from_cursor,tools_spelling_selection,switch_to_cmd_line").split (','), true);
+	RKCommonFunctions::moveContainer (m_view, "Menu", "tools", "edit", true);
+}
+
 void RKCommandEditorWindow::initializeActions (KActionCollection* ac) {
 	RK_TRACE (COMMANDEDITOR);
 
-	action_run_all = RKStandardActions::runAll (ac, "run_all", this, SLOT (runAll()));
-	action_run_selection = RKStandardActions::runSelection (ac, "run_selection", this, SLOT (runSelection()));
+	action_run_all = RKStandardActions::runAll (this, "run_all", this, SLOT (runAll()));
+	action_run_selection = RKStandardActions::runSelection (this, "run_selection", this, SLOT (runSelection()));
 	action_run_selection->setEnabled (false);
-	action_run_line = RKStandardActions::runLine (ac, "run_line", this, SLOT (runLine()));
+	action_run_line = RKStandardActions::runLine (this, "run_line", this, SLOT (runLine()));
 
-	action_help_function = RKStandardActions::functionHelp (ac, "function_reference", this, SLOT (showHelp()));
+	action_help_function = RKStandardActions::functionHelp (this, "function_reference", this, SLOT (showHelp()));
 
 	actionmenu_run_block = new KActionMenu (i18n ("Run block"), this);
 	actionmenu_run_block->setDelayed (false);	// KDE4: TODO does not work correctly in the tool bar.
@@ -161,7 +175,14 @@ void RKCommandEditorWindow::initializeActions (KActionCollection* ac) {
 	ac->addAction ("unmark_block", actionmenu_unmark_block);
 	connect (actionmenu_unmark_block->menu(), SIGNAL (aboutToShow()), this, SLOT (clearUnusedBlocks()));
 
-	QAction* action_configure = ac->addAction ("configure_commandeditor", this, SLOT (configure()));
+	action_setwd_to_script = ac->addAction ("setwd_to_script", this, SLOT (setWDToScript ()));
+	action_setwd_to_script->setText (i18n ("CD to script directory"));
+#if KDE_IS_VERSION(4,3,0)
+	action_setwd_to_script->setHelpText (i18n ("Change the working directory to the directory of this script"));
+#endif
+	action_setwd_to_script->setIcon (RKStandardIcons::getIcon (RKStandardIcons::ActionCDToScript));
+
+	KAction* action_configure = ac->addAction ("configure_commandeditor", this, SLOT (configure()));
 	action_configure->setText (i18n ("Configure Script Editor"));
 }
 
@@ -304,8 +325,7 @@ bool RKCommandEditorWindow::openURL (const KUrl &url, bool use_r_highlighting, b
 }
 
 KUrl RKCommandEditorWindow::url () {
-	RK_TRACE (COMMANDEDITOR);
-
+//	RK_TRACE (COMMANDEDITOR);
 	return (m_doc->url ());
 }
 
@@ -328,12 +348,15 @@ void RKCommandEditorWindow::setText (const QString &text) {
 
 void RKCommandEditorWindow::updateCaption (KTextEditor::Document*) {
 	RK_TRACE (COMMANDEDITOR);
-	QString name = m_doc->url ().fileName ();
-	if (name.isEmpty ()) name = m_doc->url ().prettyUrl ();
+	QString name = url ().fileName ();
+	if (name.isEmpty ()) name = url ().prettyUrl ();
 	if (name.isEmpty ()) name = i18n ("Unnamed");
 	if (isModified ()) name.append (i18n (" [modified]"));
 
 	setCaption (name);
+
+	// Well, this does not really belong, here, but needs to happen on pretty much the same occastions:
+	action_setwd_to_script->setEnabled (!url ().isEmpty ());
 }
 
 void RKCommandEditorWindow::showHelp () {
@@ -419,6 +442,18 @@ bool RKCommandEditorWindow::provideContext (unsigned int line_rev, QString *cont
 	return true;
 }
 
+void RKCommandEditorWindow::setWDToScript () {
+	RK_TRACE (COMMANDEDITOR);
+
+	RK_ASSERT (!url ().isEmpty ());
+	QString dir = url ().directory ();
+#ifdef Q_OS_WIN
+	// KURL::directory () returns a leading slash on windows as of KDElibs 4.3
+	while (dir.startsWith ('/')) dir.remove (0, 1);
+#endif
+	RKConsole::pipeUserCommand (new RCommand ("setwd (\"" + dir + "\")", RCommand::User, i18n ("cd to current script directory")));
+}
+
 void RKCommandEditorWindow::runSelection() {
 	RK_TRACE (COMMANDEDITOR);
 
@@ -601,6 +636,9 @@ RKFunctionArgHinter::RKFunctionArgHinter (RKScriptContextProvider *provider, KTe
 	arghints_popup_text = new QLabel (arghints_popup);
 	layout->addWidget (arghints_popup_text);
 	arghints_popup->hide ();
+	active = false;
+
+	connect (&updater, SIGNAL (timeout()), this, SLOT (updateArgHintWindow()));
 }
 
 RKFunctionArgHinter::~RKFunctionArgHinter () {
@@ -693,13 +731,26 @@ void RKFunctionArgHinter::tryArgHintNow () {
 	// initialize and show popup
 	arghints_popup_text->setText (effective_symbol + " (" + static_cast<RFunctionObject*> (object)->printArgs () + ')');
 	arghints_popup->resize (arghints_popup_text->sizeHint () + QSize (2, 2));
+	active = true;
+	updater.start (50);
+	updateArgHintWindow ();
+}
+
+void RKFunctionArgHinter::updateArgHintWindow () {
+	RK_TRACE (COMMANDEDITOR);
+
+	if (!active) return;
+
 	arghints_popup->move (view->mapToGlobal (view->cursorPositionCoordinates () + QPoint (0, arghints_popup->height ())));
-	arghints_popup->show ();
+	if (view->hasFocus ()) arghints_popup->show ();
+	else arghints_popup->hide ();
 }
 
 void RKFunctionArgHinter::hideArgHint () {
 	RK_TRACE (COMMANDEDITOR);
 	arghints_popup->hide ();
+	active = false;
+	updater.stop ();
 }
 
 bool RKFunctionArgHinter::eventFilter (QObject *, QEvent *e) {
