@@ -2,7 +2,7 @@
                           rkcommandeditorwindow  -  description
                              -------------------
     begin                : Mon Aug 30 2004
-    copyright            : (C) 2004, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -19,7 +19,8 @@
 
 #include <QWidget>
 #include <QVector>
-#include <qstring.h>
+#include <QTimer>
+#include <QString>
 
 #include <ktexteditor/view.h>
 #include <ktexteditor/document.h>
@@ -35,7 +36,7 @@ class QEvent;
 class QCloseEvent;
 class QFrame;
 class QLabel;
-class QAction;
+class KAction;
 class KActionMenu;
 class RKCommandEditorWindow;
 class KActionCollection;
@@ -79,6 +80,8 @@ public:
 public slots:
 	/** Internal worker function for tryArgHint () */
 	void tryArgHintNow ();
+
+	void updateArgHintWindow ();
 protected:
 	/** The (keypress) events of the view are filtered to determine, when to show / hide an argument hint */
 	bool eventFilter (QObject *, QEvent *e);
@@ -86,6 +89,9 @@ private:
 	RKScriptContextProvider *provider;
 	KTextEditor::View *view;
 
+	/** A timer to refresh the hint window periodically. This is a bit sorry, but it's really hard to find out, when the view has been moved, or gains/loses focus. While possible, this approach uses much less code. */
+	QTimer updater;
+	bool active;
 	QFrame *arghints_popup;
 	QLabel *arghints_popup_text;
 };
@@ -152,6 +158,8 @@ public:
 
 	bool provideContext (unsigned int line_rev, QString *context, int *cursor_position);
 	QString currentCompletionWord () const;
+/** reimplemented from RKMDIWindow */
+	void fixupPartGUI (bool reload);
 public slots:
 /** update Tab caption according to the current url. Display the filename-component of the URL, or - if not available - a more elaborate description of the url. Also appends a "[modified]" if appropriate */
 	void updateCaption (KTextEditor::Document* = 0);
@@ -174,6 +182,8 @@ public slots:
 
 /** selection has changed. Enable / disable actions accordingly */
 	void selectionChanged (KTextEditor::View* view);
+/** change to the directory of the current script */
+	void setWDToScript ();
 protected:
 /** reimplemented from RKMDIWindow: give the editor window a chance to object to being closed (if unsaved) */
 	void closeEvent (QCloseEvent *e);
@@ -203,9 +213,9 @@ private:
 		KTextEditor::SmartRange* range;
 		bool active;
 		KTextEditor::Attribute::Ptr attribute;
-		QAction* mark;
-		QAction* unmark;
-		QAction* run;
+		KAction* mark;
+		KAction* unmark;
+		KAction* run;
 	};
 	QVector<BlockRecord> block_records;
 	void initBlocks ();
@@ -216,11 +226,13 @@ private:
 	KActionMenu* actionmenu_unmark_block;
 	KActionMenu* actionmenu_run_block;
 
-	QAction* action_run_all;
-	QAction* action_run_selection;
-	QAction* action_run_line;
+	KAction* action_run_all;
+	KAction* action_run_selection;
+	KAction* action_run_line;
 
-	QAction* action_help_function;
+	KAction* action_setwd_to_script;
+
+	KAction* action_help_function;
 };
 
 #endif
