@@ -2,7 +2,7 @@
                           rkmdiwindow  -  description
                              -------------------
     begin                : Tue Sep 26 2006
-    copyright            : (C) 2006, 2007, 2008 by Thomas Friedrichsmeier
+    copyright            : (C) 2006, 2007, 2008, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -28,6 +28,12 @@ class QPaintEvent;
 class RKWorkplace;
 class RKToolWindowBar;
 
+class RKMDIStandardActionClient : public KXMLGUIClient {
+public:
+	RKMDIStandardActionClient ();
+	~RKMDIStandardActionClient ();
+};
+
 /** Base class for rkward document mdi windows */
 class RKMDIWindow : public QFrame {
 	Q_OBJECT
@@ -62,6 +68,9 @@ protected:
 @param type Type of window (see RKMDIWindow::Type).*/
 	RKMDIWindow (QWidget *parent, int type, bool tool_window=false, const char *name=0);
 	virtual ~RKMDIWindow ();
+public slots:
+/** Reimplemented from QWidget::setCaption () to emit the signal captionChanged () when the caption is changed. */
+	void setCaption (const QString &caption);
 public:
 /** @returns true, if the window's document was modified (and would need to be saved) */
 	virtual bool isModified () { return false; };
@@ -74,8 +83,6 @@ public:
 /** This is used in RKWorkplace::saveWorkplace () to save the info about the workplace. Make sure to add corresponding code to RKWorkplace::restoreWorkplace (), so your window(s) get restored when loading a Workspace
 @returns An internal descriptive string. */
 	virtual QString getDescription () { return QString (); };
-/** Reimplemented from QWidget::setCaption () to emit the signal captionChanged () when the caption is changed. */
-	void setCaption (const QString &caption);
 /** Is this window attached (or detached)?
 @returns true if attached, false if detached */
 	bool isAttached () const { return (state == Attached); };
@@ -97,6 +104,11 @@ public:
 	bool acceptsEventsFor (QObject *object);
 /** Whether the window is active. This seems to be more reliable than hasFocus () */
 	bool isActive ();
+/** Returns a pointer to an action collection suitable to place RKStandardAction in. This collection (and the corresponding KXMLGUIClient) is created on the fly. */
+	KActionCollection *standardActionCollection ();
+/** Used to update the part gui after some aspects of it have changed (most importantly, after keyboard shortcuts were changed). This also takes care of updating the child clients. Reimplemented
+in some classes to re-apply customizations of the part GUI. */
+	virtual void fixupPartGUI (bool reload);
 signals:
 /** This signal is emitted, whenever the window caption was changed.
 @param RKMDIWindow* a pointer to this window */
@@ -121,6 +133,7 @@ friend class RKToolWindowBar;
 	State state;
 	RKToolWindowBar *tool_window_bar;
 	bool active;
+	RKMDIStandardActionClient *standard_client;
 };
 
 #endif
