@@ -2,7 +2,7 @@
                           rktoplevelwindowgui  -  description
                              -------------------
     begin                : Tue Apr 24 2007
-    copyright            : (C) 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,6 +22,8 @@
 #include <kaboutapplicationdialog.h>
 #include <kcmdlineargs.h>
 #include <kactioncollection.h>
+#include <kxmlguifactory.h>
+#include <kshortcutsdialog.h>
 
 #include <QWhatsThis>
 
@@ -36,6 +38,7 @@
 #include "../windows/rkhelpsearchwindow.h"
 #include "../windows/rkmdiwindow.h"
 #include "../misc/rkstandardicons.h"
+#include "../plugin/rkcomponentmap.h"
 #include "../rbackend/rinterface.h"
 #include "../rkglobals.h"
 #include "../rkward.h"
@@ -66,7 +69,7 @@ RKTopLevelWindowGUI::RKTopLevelWindowGUI (QWidget *for_window) : QObject (for_wi
 	show_rkward_help->setStatusTip (i18n ("Show help on RKWard"));
 
 	// window menu
-	QAction *action;
+	KAction *action;
 	action = actionCollection ()->addAction ("window_show_workspace", this, SLOT(toggleWorkspace()));
 	action->setText (i18n ("Show/Hide Workspace Browser"));
 	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowWorkspaceBrowser));
@@ -98,12 +101,28 @@ RKTopLevelWindowGUI::RKTopLevelWindowGUI (QWidget *for_window) : QObject (for_wi
 	action = actionCollection ()->addAction ("output_show", this, SLOT (slotOutputShow()));
 	action->setText (i18n ("Show &Output"));
 	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowOutput));
+
+	// settings
+	KStandardAction::keyBindings (this, SLOT (configureShortcuts ()), actionCollection ());
 }
 
 RKTopLevelWindowGUI::~RKTopLevelWindowGUI () {
 	RK_TRACE (APP);
 }
 
+void RKTopLevelWindowGUI::configureShortcuts () {
+	RK_TRACE (APP);
+
+	KMessageBox::information (for_window, i18n ("For technical reasons, the following dialog allows you to configure the keyboard shortcuts only for those parts of RKWard that are currently active.\n\nTherefore, if you want to configure keyboard shortcuts e.g. for use inside the script editor, you need to open a script editor window, and activate it."), i18n ("Note"), "configure_shortcuts_kparts");
+
+	KShortcutsDialog dlg (KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, qobject_cast<QWidget*> (parent()));
+	foreach (KXMLGUIClient *client, factory ()->clients ()) {
+		if (client && !client->xmlFile ().isEmpty ()) dlg.addCollection (client->actionCollection());
+	}
+	dlg.addCollection (RKComponentMap::getMap ()->actionCollection (), i18n ("RKWard Plugins"));
+	dlg.configure (true);
+}
+
 void RKTopLevelWindowGUI::invokeRHelp () {
 	RK_TRACE (APP);
 
@@ -121,7 +140,9 @@ void RKTopLevelWindowGUI::reportRKWardBug () {
 	RK_TRACE (APP);
 
 // TOOD: something pretty
-	KMessageBox::information (for_window, i18n ("Please submit your bug reports or wishes at http://sourceforge.net/tracker/?group_id=50231&atid=459007 or send email to rkward-devel@lists.sourceforge.net"));
+	KMessageBox::information (for_window, i18n ("<p>Please submit your bug reports or wishes at <a href=\"%1\">%1</a> or send email to <a href=\"mailto:%2\">%2</a>.</p>"
+							, QString ("http://sourceforge.net/tracker/?group_id=50231&atid=459007"), QString ("rkward-devel@lists.sourceforge.net")),
+							i18n ("Reporting bugs in RKWard"), QString (), KMessageBox::Notify | KMessageBox::AllowLink);
 }
 
 void RKTopLevelWindowGUI::showAboutApplication () {
