@@ -2,7 +2,7 @@
                           rktoplevelwindowgui  -  description
                              -------------------
     begin                : Tue Apr 24 2007
-    copyright            : (C) 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -63,6 +63,10 @@ public slots:
 	void reportRKWardBug ();
 	/** not quite sure, why I have to reimplement this from KMainWindow */
 	void showAboutApplication ();
+
+	// settings menu
+	/** configure key bindings. Implemented to show notice before the actual dialog. */
+	void configureShortcuts ();
 private:
 	QWidget *for_window;
 	void toggleToolView (RKMDIWindow *tool_window);
